@@ -114,8 +114,8 @@ impl EditAgent {
         let (events_tx, events_rx) = mpsc::unbounded();
         let conversation = conversation.clone();
         let output = cx.spawn(async move |cx| {
-            let snapshot = buffer.read_with(cx, |buffer, _| buffer.snapshot())?;
-            let path = cx.update(|cx| snapshot.resolve_file_path(true, cx))?;
+            let snapshot = buffer.read_with(cx, |buffer, _| buffer.snapshot());
+            let path = cx.update(|cx| snapshot.resolve_file_path(true, cx));
             let prompt = CreateFilePromptTemplate {
                 path,
                 edit_description,
@@ -148,7 +148,7 @@ impl EditAgent {
         let this = self.clone();
         let task = cx.spawn(async move |cx| {
             this.action_log
-                .update(cx, |log, cx| log.buffer_created(buffer.clone(), cx))?;
+                .update(cx, |log, cx| log.buffer_created(buffer.clone(), cx));
             this.overwrite_with_chunks_internal(buffer, parse_rx, output_events_tx, cx)
                 .await?;
             parse_task.await
@@ -182,7 +182,7 @@ impl EditAgent {
                     Anchor::min_max_range_for_buffer(buffer.read(cx).remote_id()),
                 ))
                 .ok();
-        })?;
+        });
 
         while let Some(event) = parse_rx.next().await {
             match event? {
@@ -203,7 +203,7 @@ impl EditAgent {
                             )
                         });
                         buffer.read(cx).remote_id()
-                    })?;
+                    });
                     output_events_tx
                         .unbounded_send(EditAgentOutputEvent::Edited(
                             Anchor::min_max_range_for_buffer(buffer_id),
@@ -231,8 +231,8 @@ impl EditAgent {
         let conversation = conversation.clone();
         let edit_format = self.edit_format;
         let output = cx.spawn(async move |cx| {
-            let snapshot = buffer.read_with(cx, |buffer, _| buffer.snapshot())?;
-            let path = cx.update(|cx| snapshot.resolve_file_path(true, cx))?;
+            let snapshot = buffer.read_with(cx, |buffer, _| buffer.snapshot());
+            let path = cx.update(|cx| snapshot.resolve_file_path(true, cx));
             let prompt = match edit_format {
                 EditFormat::XmlTags => EditFileXmlPromptTemplate {
                     path,
@@ -263,7 +263,7 @@ impl EditAgent {
         cx: &mut AsyncApp,
     ) -> Result<EditAgentOutput> {
         self.action_log
-            .update(cx, |log, cx| log.buffer_read(buffer.clone(), cx))?;
+            .update(cx, |log, cx| log.buffer_read(buffer.clone(), cx));
 
         let (output, edit_events) = Self::parse_edit_chunks(edit_chunks, self.edit_format, cx);
         let mut edit_events = edit_events.peekable();
@@ -274,7 +274,7 @@ impl EditAgent {
                 continue;
             };
 
-            let snapshot = buffer.read_with(cx, |buffer, _| buffer.snapshot())?;
+            let snapshot = buffer.read_with(cx, |buffer, _| buffer.snapshot());
 
             // Resolve the old text in the background, updating the agent
             // location as we keep refining which range it corresponds to.
@@ -292,7 +292,7 @@ impl EditAgent {
                             }),
                             cx,
                         );
-                    })?;
+                    });
                     output_events
                         .unbounded_send(EditAgentOutputEvent::ResolvingEditRange(old_range))
                         .ok();
@@ -375,7 +375,7 @@ impl EditAgent {
                         );
                     });
                     (min_edit_start, max_edit_end)
-                })?;
+                });
                 output_events
                     .unbounded_send(EditAgentOutputEvent::Edited(min_edit_start..max_edit_end))
                     .ok();
@@ -726,13 +726,16 @@ impl EditAgent {
             thread_id: conversation.thread_id,
             prompt_id: conversation.prompt_id,
             intent: Some(intent),
-            mode: conversation.mode,
             messages: conversation.messages,
             tool_choice,
             tools,
             stop: Vec::new(),
             temperature: None,
             thinking_allowed: true,
+            // Bypass the rate limiter for nested requests (edit agent requests spawned
+            // from within a tool call) to avoid deadlocks when multiple subagents try
+            // to use edit_file simultaneously.
+            bypass_rate_limit: true,
         };
 
         Ok(self.model.stream_completion_text(request, cx).await?.stream)
