@@ -1,5 +1,5 @@
 use crate::ProjectSnapshot;
-use agent_settings::{AgentProfileId, CompletionMode};
+use agent_settings::AgentProfileId;
 use anyhow::Result;
 use chrono::{DateTime, Utc};
 use gpui::SharedString;
@@ -37,8 +37,6 @@ pub struct SerializedThread {
     #[serde(default)]
     pub model: Option<SerializedLanguageModel>,
     #[serde(default)]
-    pub completion_mode: Option<CompletionMode>,
-    #[serde(default)]
     pub tool_use_limit_reached: bool,
     #[serde(default)]
     pub profile: Option<AgentProfileId>,
@@ -186,7 +184,6 @@ impl LegacySerializedThread {
             request_token_usage: Vec::new(),
             detailed_summary_state: DetailedSummaryState::default(),
             model: None,
-            completion_mode: None,
             tool_use_limit_reached: false,
             profile: None,
         }
@@ -275,7 +272,6 @@ mod tests {
                 request_token_usage: vec![],
                 detailed_summary_state: DetailedSummaryState::default(),
                 model: None,
-                completion_mode: None,
                 tool_use_limit_reached: false,
                 profile: None
             }
@@ -341,7 +337,6 @@ mod tests {
             request_token_usage: vec![],
             detailed_summary_state: DetailedSummaryState::default(),
             model: None,
-            completion_mode: None,
             tool_use_limit_reached: false,
             profile: None,
         });
@@ -393,7 +388,6 @@ mod tests {
                 request_token_usage: vec![],
                 detailed_summary_state: DetailedSummaryState::default(),
                 model: None,
-                completion_mode: None,
                 tool_use_limit_reached: false,
                 profile: None
             }
