@@ -14,12 +14,14 @@ mod open_tool;
 mod read_file_tool;
 mod restore_file_from_disk_tool;
 mod save_file_tool;
-
+mod subagent_tool;
 mod terminal_tool;
 mod thinking_tool;
 mod web_search_tool;
 
 use crate::AgentTool;
+use feature_flags::{FeatureFlagAppExt, SubagentsFeatureFlag};
+use gpui::App;
 use language_model::{LanguageModelRequestTool, LanguageModelToolSchemaFormat};
 
 pub use context_server_registry::*;
@@ -38,7 +40,7 @@ pub use open_tool::*;
 pub use read_file_tool::*;
 pub use restore_file_from_disk_tool::*;
 pub use save_file_tool::*;
-
+pub use subagent_tool::*;
 pub use terminal_tool::*;
 pub use thinking_tool::*;
 pub use web_search_tool::*;
@@ -46,8 +48,8 @@ pub use web_search_tool::*;
 macro_rules! tools {
     ($($tool:ty),* $(,)?) => {
         /// A list of all built-in tool names
-        pub fn supported_built_in_tool_names(provider: Option<language_model::LanguageModelProviderId>) -> impl Iterator<Item = String> {
-            [
+        pub fn supported_built_in_tool_names(provider: Option<language_model::LanguageModelProviderId>, cx: &App) -> Vec<String> {
+            let mut tools: Vec<String> = [
                 $(
                     (if let Some(provider) = provider.as_ref() {
                         <$tool>::supports_provider(provider)
@@ -59,6 +61,13 @@ macro_rules! tools {
             ]
             .into_iter()
             .flatten()
+            .collect();
+
+            if !cx.has_flag::<SubagentsFeatureFlag>() {
+                tools.retain(|name| name != SubagentTool::name());
+            }
+
+            tools
         }
 
         /// A list of all built-in tools
@@ -96,6 +105,7 @@ tools! {
     ReadFileTool,
     RestoreFileFromDiskTool,
     SaveFileTool,
+    SubagentTool,
     TerminalTool,
     ThinkingTool,
     WebSearchTool,
