@@ -1,6 +1,7 @@
 use action_log::ActionLog;
 use agent_client_protocol::{self as acp, ToolCallUpdateFields};
 use anyhow::{Context as _, Result, anyhow};
+use futures::FutureExt as _;
 use gpui::{App, Entity, SharedString, Task, WeakEntity};
 use indoc::formatdoc;
 use language::Point;
@@ -64,6 +65,14 @@ impl ReadFileTool {
             action_log,
         }
     }
+
+    pub fn with_thread(&self, new_thread: WeakEntity<Thread>) -> Self {
+        Self {
+            thread: new_thread,
+            project: self.project.clone(),
+            action_log: self.action_log.clone(),
+        }
+    }
 }
 
 impl AgentTool for ReadFileTool {
@@ -167,14 +176,14 @@ impl AgentTool for ReadFileTool {
                         self.project.update(cx, |project, cx| {
                             project.open_image(project_path.clone(), cx)
                         })
-                    })?
+                    })
                     .await?;
 
                 let image =
-                    image_entity.read_with(cx, |image_item, _| Arc::clone(&image_item.image))?;
+                    image_entity.read_with(cx, |image_item, _| Arc::clone(&image_item.image));
 
                 let language_model_image = cx
-                    .update(|cx| LanguageModelImage::from_image(image, cx))?
+                    .update(|cx| LanguageModelImage::from_image(image, cx))
                     .await
                     .context("processing image")?;
 
@@ -192,26 +201,31 @@ impl AgentTool for ReadFileTool {
         let action_log = self.action_log.clone();
 
         cx.spawn(async move |cx| {
-            let buffer = cx
-                .update(|cx| {
-                    project.update(cx, |project, cx| {
-                        project.open_buffer(project_path.clone(), cx)
-                    })
-                })?
-                .await?;
+            let open_buffer_task = cx.update(|cx| {
+                project.update(cx, |project, cx| {
+                    project.open_buffer(project_path.clone(), cx)
+                })
+            });
+
+            let buffer = futures::select! {
+                result = open_buffer_task.fuse() => result?,
+                _ = event_stream.cancelled_by_user().fuse() => {
+                    anyhow::bail!("File read cancelled by user");
+                }
+            };
             if buffer.read_with(cx, |buffer, _| {
                 buffer
                     .file()
                     .as_ref()
                     .is_none_or(|file| !file.disk_state().exists())
-            })? {
+            }) {
                 anyhow::bail!("{file_path} not found");
             }
 
             // Record the file read time and mtime
             if let Some(mtime) = buffer.read_with(cx, |buffer, _| {
                 buffer.file().and_then(|file| file.disk_state().mtime())
-            })? {
+            }) {
                 self.thread
                     .update(cx, |thread, _| {
                         thread.file_read_times.insert(abs_path.to_path_buf(), mtime);
@@ -239,11 +253,11 @@ impl AgentTool for ReadFileTool {
                     let start = buffer.anchor_before(Point::new(start_row, 0));
                     let end = buffer.anchor_before(Point::new(end_row, 0));
                     buffer.text_for_range(start..end).collect::<String>()
-                })?;
+                });
 
                 action_log.update(cx, |log, cx| {
                     log.buffer_read(buffer.clone(), cx);
-                })?;
+                });
 
                 Ok(result.into())
             } else {
@@ -257,7 +271,7 @@ impl AgentTool for ReadFileTool {
 
                 action_log.update(cx, |log, cx| {
                     log.buffer_read(buffer.clone(), cx);
-                })?;
+                });
 
                 if buffer_content.is_outline {
                     Ok(formatdoc! {"
@@ -297,11 +311,18 @@ impl AgentTool for ReadFileTool {
                         acp::ToolCallContent::Content(acp::Content::new(markdown)),
                     ]));
                 }
-            })?;
+            });
 
             result
         })
     }
+
+    fn rebind_thread(
+        &self,
+        new_thread: WeakEntity<Thread>,
+    ) -> Option<std::sync::Arc<dyn crate::AnyAgentTool>> {
+        Some(self.with_thread(new_thread).erase())
+    }
 }
 
 #[cfg(test)]
