@@ -1,7 +1,8 @@
 use std::{cell::RefCell, ops::Range, rc::Rc};
 
+use super::thread_history::AcpThreadHistory;
 use acp_thread::{AcpThread, AgentThreadEntry};
-use agent::HistoryStore;
+use agent::ThreadStore;
 use agent_client_protocol::{self as acp, ToolCallId};
 use collections::HashMap;
 use editor::{Editor, EditorMode, MinimapVisibility, SizingBehavior};
@@ -23,7 +24,8 @@ use crate::acp::message_editor::{MessageEditor, MessageEditorEvent};
 pub struct EntryViewState {
     workspace: WeakEntity<Workspace>,
     project: WeakEntity<Project>,
-    history_store: Entity<HistoryStore>,
+    thread_store: Option<Entity<ThreadStore>>,
+    history: WeakEntity<AcpThreadHistory>,
     prompt_store: Option<Entity<PromptStore>>,
     entries: Vec<Entry>,
     prompt_capabilities: Rc<RefCell<acp::PromptCapabilities>>,
@@ -35,7 +37,8 @@ impl EntryViewState {
     pub fn new(
         workspace: WeakEntity<Workspace>,
         project: WeakEntity<Project>,
-        history_store: Entity<HistoryStore>,
+        thread_store: Option<Entity<ThreadStore>>,
+        history: WeakEntity<AcpThreadHistory>,
         prompt_store: Option<Entity<PromptStore>>,
         prompt_capabilities: Rc<RefCell<acp::PromptCapabilities>>,
         available_commands: Rc<RefCell<Vec<acp::AvailableCommand>>>,
@@ -44,7 +47,8 @@ impl EntryViewState {
         Self {
             workspace,
             project,
-            history_store,
+            thread_store,
+            history,
             prompt_store,
             entries: Vec::new(),
             prompt_capabilities,
@@ -85,7 +89,8 @@ impl EntryViewState {
                         let mut editor = MessageEditor::new(
                             self.workspace.clone(),
                             self.project.clone(),
-                            self.history_store.clone(),
+                            self.thread_store.clone(),
+                            self.history.clone(),
                             self.prompt_store.clone(),
                             self.prompt_capabilities.clone(),
                             self.available_commands.clone(),
@@ -396,12 +401,11 @@ fn diff_editor_text_style_refinement(cx: &mut App) -> TextStyleRefinement {
 
 #[cfg(test)]
 mod tests {
-    use std::{path::Path, rc::Rc};
+    use std::path::Path;
+    use std::rc::Rc;
 
     use acp_thread::{AgentConnection, StubAgentConnection};
-    use agent::HistoryStore;
     use agent_client_protocol as acp;
-    use assistant_text_thread::TextThreadStore;
     use buffer_diff::{DiffHunkStatus, DiffHunkStatusKind};
     use editor::RowInfo;
     use fs::FakeFs;
@@ -452,14 +456,16 @@ mod tests {
             connection.send_update(session_id, acp::SessionUpdate::ToolCall(tool_call), cx)
         });
 
-        let text_thread_store = cx.new(|cx| TextThreadStore::fake(project.clone(), cx));
-        let history_store = cx.new(|cx| HistoryStore::new(text_thread_store, cx));
+        let thread_store = None;
+        let history = cx
+            .update(|window, cx| cx.new(|cx| crate::acp::AcpThreadHistory::new(None, window, cx)));
 
         let view_state = cx.new(|_cx| {
             EntryViewState::new(
                 workspace.downgrade(),
                 project.downgrade(),
-                history_store,
+                thread_store,
+                history.downgrade(),
                 None,
                 Default::default(),
                 Default::default(),
@@ -471,7 +477,7 @@ mod tests {
             view_state.sync_entry(0, &thread, window, cx)
         });
 
-        let diff = thread.read_with(cx, |thread, _cx| {
+        let diff = thread.read_with(cx, |thread, _| {
             thread
                 .entries()
                 .get(0)
