@@ -6,7 +6,6 @@ use uuid::Uuid;
 use cloud_llm_client::CompletionIntent;
 use collections::HashSet;
 use editor::{Anchor, AnchorRangeExt, MultiBuffer, MultiBufferSnapshot, ToOffset as _, ToPoint};
-use feature_flags::{FeatureFlagAppExt as _, InlineAssistantUseToolFeatureFlag};
 use futures::{
     SinkExt, Stream, StreamExt, TryStreamExt as _,
     channel::mpsc,
@@ -401,7 +400,6 @@ impl CodegenAlternative {
 
     pub fn use_streaming_tools(model: &dyn LanguageModel, cx: &App) -> bool {
         model.supports_streaming_tools()
-            && cx.has_flag::<InlineAssistantUseToolFeatureFlag>()
             && AgentSettings::get_global(cx).inline_assistant_use_streaming_tools
     }
 
@@ -540,13 +538,13 @@ impl CodegenAlternative {
                 thread_id: None,
                 prompt_id: None,
                 intent: Some(CompletionIntent::InlineAssist),
-                mode: None,
                 tools,
                 tool_choice,
                 stop: Vec::new(),
                 temperature,
                 messages,
                 thinking_allowed: false,
+                bypass_rate_limit: false,
             }
         }))
     }
@@ -619,13 +617,13 @@ impl CodegenAlternative {
                 thread_id: None,
                 prompt_id: None,
                 intent: Some(CompletionIntent::InlineAssist),
-                mode: None,
                 tools: Vec::new(),
                 tool_choice: None,
                 stop: Vec::new(),
                 temperature,
                 messages: vec![request_message],
                 thinking_allowed: false,
+                bypass_rate_limit: false,
             }
         }))
     }
