@@ -1,21 +1,17 @@
 mod acp_onboarding_modal;
 mod agent_notification;
-mod burn_mode_tooltip;
 mod claude_code_onboarding_modal;
 mod end_trial_upsell;
 mod hold_for_default;
 mod mention_crease;
 mod model_selector_components;
 mod onboarding_modal;
-mod usage_callout;
 
 pub use acp_onboarding_modal::*;
 pub use agent_notification::*;
-pub use burn_mode_tooltip::*;
 pub use claude_code_onboarding_modal::*;
 pub use end_trial_upsell::*;
 pub use hold_for_default::*;
 pub use mention_crease::*;
 pub use model_selector_components::*;
 pub use onboarding_modal::*;
-pub use usage_callout::*;
