@@ -73,24 +73,27 @@ impl AskPassDelegate {
 
 pub struct AskPassSession {
     #[cfg(target_os = "windows")]
-    secret: std::sync::Arc<OnceLock<EncryptedPassword>>,
+    secret: std::sync::Arc<std::sync::Mutex<Option<EncryptedPassword>>>,
     askpass_task: PasswordProxy,
     askpass_opened_rx: Option<oneshot::Receiver<()>>,
     askpass_kill_master_rx: Option<oneshot::Receiver<()>>,
+    executor: BackgroundExecutor,
 }
 
-#[cfg(not(target_os = "windows"))]
-const ASKPASS_SCRIPT_NAME: &str = "askpass.sh";
-#[cfg(target_os = "windows")]
-const ASKPASS_SCRIPT_NAME: &str = "askpass.ps1";
+const ASKPASS_SCRIPT_NAME: &str = if cfg!(target_os = "windows") {
+    "askpass.ps1"
+} else {
+    "askpass.sh"
+};
 
 impl AskPassSession {
     /// This will create a new AskPassSession.
     /// You must retain this session until the master process exits.
     #[must_use]
-    pub async fn new(executor: &BackgroundExecutor, mut delegate: AskPassDelegate) -> Result<Self> {
+    pub async fn new(executor: BackgroundExecutor, mut delegate: AskPassDelegate) -> Result<Self> {
         #[cfg(target_os = "windows")]
-        let secret = std::sync::Arc::new(OnceLock::new());
+        let secret = std::sync::Arc::new(std::sync::Mutex::new(None));
+
         let (askpass_opened_tx, askpass_opened_rx) = oneshot::channel::<()>();
 
         let askpass_opened_tx = Arc::new(Mutex::new(Some(askpass_opened_tx)));
@@ -98,11 +101,11 @@ impl AskPassSession {
         let (askpass_kill_master_tx, askpass_kill_master_rx) = oneshot::channel::<()>();
         let kill_tx = Arc::new(Mutex::new(Some(askpass_kill_master_tx)));
 
-        #[cfg(target_os = "windows")]
-        let askpass_secret = secret.clone();
         let get_password = {
             let executor = executor.clone();
 
+            #[cfg(target_os = "windows")]
+            let askpass_secret = secret.clone();
             move |prompt| {
                 let prompt = delegate.ask_password(prompt);
                 let kill_tx = kill_tx.clone();
@@ -116,7 +119,7 @@ impl AskPassSession {
                     if let Some(password) = prompt.await {
                         #[cfg(target_os = "windows")]
                         {
-                            _ = askpass_secret.set(password.clone());
+                            askpass_secret.lock().unwrap().replace(password.clone());
                         }
                         ControlFlow::Continue(Ok(password))
                     } else {
@@ -137,6 +140,7 @@ impl AskPassSession {
             askpass_task,
             askpass_kill_master_rx: Some(askpass_kill_master_rx),
             askpass_opened_rx: Some(askpass_opened_rx),
+            executor,
         })
     }
 
@@ -152,6 +156,7 @@ impl AskPassSession {
             .askpass_kill_master_rx
             .take()
             .expect("Only call run once");
+        let executor = self.executor.clone();
 
         select_biased! {
             _ = askpass_opened_rx.fuse() => {
@@ -160,7 +165,7 @@ impl AskPassSession {
                 AskPassResult::CancelledByUser
             }
 
-            _ = futures::FutureExt::fuse(smol::Timer::after(connection_timeout)) => {
+            _ = futures::FutureExt::fuse(executor.timer(connection_timeout)) => {
                 AskPassResult::Timedout
             }
         }
@@ -169,7 +174,7 @@ impl AskPassSession {
     /// This will return the password that was last set by the askpass script.
     #[cfg(target_os = "windows")]
     pub fn get_password(&self) -> Option<EncryptedPassword> {
-        self.secret.get().cloned()
+        self.secret.lock().ok()?.clone()
     }
 
     pub fn script_path(&self) -> impl AsRef<OsStr> {
