@@ -188,7 +188,7 @@ impl SlashCommand for DiagnosticsSlashCommand {
         let path_style = project.read(cx).path_style(cx);
         let options = Options::parse(arguments, path_style);
 
-        let task = collect_diagnostics(project.clone(), options, cx);
+        let task = collect_diagnostics_output(project.clone(), options, cx);
 
         window.spawn(cx, async move |_| {
             task.await?
@@ -198,10 +198,10 @@ impl SlashCommand for DiagnosticsSlashCommand {
     }
 }
 
-#[derive(Default)]
-struct Options {
-    include_warnings: bool,
-    path_matcher: Option<PathMatcher>,
+pub struct Options {
+    pub include_errors: bool,
+    pub include_warnings: bool,
+    pub path_matcher: Option<PathMatcher>,
 }
 
 const INCLUDE_WARNINGS_ARGUMENT: &str = "--include-warnings";
@@ -218,6 +218,7 @@ impl Options {
             }
         }
         Self {
+            include_errors: true,
             include_warnings,
             path_matcher,
         }
@@ -228,7 +229,7 @@ impl Options {
     }
 }
 
-fn collect_diagnostics(
+pub fn collect_diagnostics_output(
     project: Entity<Project>,
     options: Options,
     cx: &mut App,
@@ -282,11 +283,17 @@ fn collect_diagnostics(
                 continue;
             }
 
-            project_summary.error_count += summary.error_count;
+            let has_errors = options.include_errors && summary.error_count > 0;
+            let has_warnings = options.include_warnings && summary.warning_count > 0;
+            if !has_errors && !has_warnings {
+                continue;
+            }
+
+            if options.include_errors {
+                project_summary.error_count += summary.error_count;
+            }
             if options.include_warnings {
                 project_summary.warning_count += summary.warning_count;
-            } else if summary.error_count == 0 {
-                continue;
             }
 
             let last_end = output.text.len();
@@ -300,8 +307,13 @@ fn collect_diagnostics(
                 .await
                 .log_err()
             {
-                let snapshot = cx.read_entity(&buffer, |buffer, _| buffer.snapshot())?;
-                collect_buffer_diagnostics(&mut output, &snapshot, options.include_warnings);
+                let snapshot = cx.read_entity(&buffer, |buffer, _| buffer.snapshot());
+                collect_buffer_diagnostics(
+                    &mut output,
+                    &snapshot,
+                    options.include_warnings,
+                    options.include_errors,
+                );
             }
 
             if !glob_is_exact_file_match {
@@ -358,10 +370,11 @@ pub fn collect_buffer_diagnostics(
     output: &mut SlashCommandOutput,
     snapshot: &BufferSnapshot,
     include_warnings: bool,
+    include_errors: bool,
 ) {
     for (_, group) in snapshot.diagnostic_groups(None) {
         let entry = &group.entries[group.primary_ix];
-        collect_diagnostic(output, entry, snapshot, include_warnings)
+        collect_diagnostic(output, entry, snapshot, include_warnings, include_errors)
     }
 }
 
@@ -370,6 +383,7 @@ fn collect_diagnostic(
     entry: &DiagnosticEntryRef<'_, Anchor>,
     snapshot: &BufferSnapshot,
     include_warnings: bool,
+    include_errors: bool,
 ) {
     const EXCERPT_EXPANSION_SIZE: u32 = 2;
     const MAX_MESSAGE_LENGTH: usize = 2000;
@@ -381,7 +395,12 @@ fn collect_diagnostic(
             }
             ("warning", IconName::Warning)
         }
-        DiagnosticSeverity::ERROR => ("error", IconName::XCircle),
+        DiagnosticSeverity::ERROR => {
+            if !include_errors {
+                return;
+            }
+            ("error", IconName::XCircle)
+        }
         _ => return,
     };
     let prev_len = output.text.len();
