@@ -436,7 +436,7 @@ impl AutoUpdater {
                 .0
                 .clone()
                 .context("auto-update not initialized")
-        })??;
+        })?;
 
         set_status("Fetching remote server release", cx);
         let release = Self::get_release_asset(
@@ -456,7 +456,7 @@ impl AutoUpdater {
         let version_path = platform_dir.join(format!("{}.gz", release.version));
         smol::fs::create_dir_all(&platform_dir).await.ok();
 
-        let client = this.read_with(cx, |this, _| this.client.http_client())?;
+        let client = this.read_with(cx, |this, _| this.client.http_client());
 
         if smol::fs::metadata(&version_path).await.is_err() {
             log::info!(
@@ -482,7 +482,7 @@ impl AutoUpdater {
                 .0
                 .clone()
                 .context("auto-update not initialized")
-        })??;
+        })?;
 
         let release =
             Self::get_release_asset(&this, channel, version, "zed-remote-server", os, arch, cx)
@@ -500,7 +500,7 @@ impl AutoUpdater {
         arch: &str,
         cx: &mut AsyncApp,
     ) -> Result<ReleaseAsset> {
-        let client = this.read_with(cx, |this, _| this.client.clone())?;
+        let client = this.read_with(cx, |this, _| this.client.clone());
 
         let (system_id, metrics_id, is_staff) = if client.telemetry().metrics_enabled() {
             (
@@ -563,7 +563,7 @@ impl AutoUpdater {
                     this.status.clone(),
                     ReleaseChannel::try_global(cx).unwrap_or(ReleaseChannel::Stable),
                 )
-            })?;
+            });
 
         Self::check_dependencies()?;
 
@@ -571,12 +571,12 @@ impl AutoUpdater {
             this.status = AutoUpdateStatus::Checking;
             log::info!("Auto Update: checking for updates");
             cx.notify();
-        })?;
+        });
 
         let fetched_release_data =
             Self::get_release_asset(&this, release_channel, None, "zed", OS, ARCH, cx).await?;
         let fetched_version = fetched_release_data.clone().version;
-        let app_commit_sha = cx.update(|cx| AppCommitSha::try_global(cx).map(|sha| sha.full()));
+        let app_commit_sha = Ok(cx.update(|cx| AppCommitSha::try_global(cx).map(|sha| sha.full())));
         let newer_version = Self::check_if_fetched_version_is_newer(
             release_channel,
             app_commit_sha,
@@ -586,7 +586,7 @@ impl AutoUpdater {
         )?;
 
         let Some(newer_version) = newer_version else {
-            return this.update(cx, |this, cx| {
+            this.update(cx, |this, cx| {
                 let status = match previous_status {
                     AutoUpdateStatus::Updated { .. } => previous_status,
                     _ => AutoUpdateStatus::Idle,
@@ -594,6 +594,7 @@ impl AutoUpdater {
                 this.status = status;
                 cx.notify();
             });
+            return Ok(());
         };
 
         this.update(cx, |this, cx| {
@@ -601,7 +602,7 @@ impl AutoUpdater {
                 version: newer_version.clone(),
             };
             cx.notify();
-        })?;
+        });
 
         let installer_dir = InstallerDir::new().await?;
         let target_path = Self::target_path(&installer_dir).await?;
@@ -612,11 +613,11 @@ impl AutoUpdater {
                 version: newer_version.clone(),
             };
             cx.notify();
-        })?;
+        });
 
         let new_binary_path = Self::install_release(installer_dir, target_path, cx).await?;
         if let Some(new_binary_path) = new_binary_path {
-            cx.update(|cx| cx.set_restart_path(new_binary_path))?;
+            cx.update(|cx| cx.set_restart_path(new_binary_path));
         }
 
         this.update(cx, |this, cx| {
@@ -626,7 +627,8 @@ impl AutoUpdater {
                 version: newer_version,
             };
             cx.notify();
-        })
+        });
+        Ok(())
     }
 
     fn check_if_fetched_version_is_newer(
@@ -807,9 +809,9 @@ async fn install_release_linux(
     downloaded_tar_gz: PathBuf,
     cx: &AsyncApp,
 ) -> Result<Option<PathBuf>> {
-    let channel = cx.update(|cx| ReleaseChannel::global(cx).dev_name())?;
+    let channel = cx.update(|cx| ReleaseChannel::global(cx).dev_name());
     let home_dir = PathBuf::from(env::var("HOME").context("no HOME env var set")?);
-    let running_app_path = cx.update(|cx| cx.app_path())??;
+    let running_app_path = cx.update(|cx| cx.app_path())?;
 
     let extracted = temp_dir.path().join("zed");
     fs::create_dir_all(&extracted)
@@ -874,7 +876,7 @@ async fn install_release_macos(
     downloaded_dmg: PathBuf,
     cx: &AsyncApp,
 ) -> Result<Option<PathBuf>> {
-    let running_app_path = cx.update(|cx| cx.app_path())??;
+    let running_app_path = cx.update(|cx| cx.app_path())?;
     let running_app_filename = running_app_path
         .file_name()
         .with_context(|| format!("invalid running app path {running_app_path:?}"))?;
