@@ -800,7 +800,6 @@ impl RandomizedTest for ProjectCollaborationTest {
                     assert!(
                         buffer
                             .read_with(&cx, |buffer, _| { buffer.saved_version().to_owned() })
-                            .expect("App should not be dropped")
                             .observed_all(&requested_version)
                     );
                     anyhow::Ok(())
@@ -1111,7 +1110,8 @@ impl RandomizedTest for ProjectCollaborationTest {
                             let fs = fs.clone();
                             move |_, cx| {
                                 let background = cx.background_executor();
-                                let mut rng = background.rng();
+                                let rng = background.rng();
+                                let mut rng = rng.lock();
                                 let count = rng.random_range::<usize, _>(1..3);
                                 let files = fs.as_fake().files();
                                 let files = (0..count)
@@ -1137,7 +1137,8 @@ impl RandomizedTest for ProjectCollaborationTest {
                                 move |_, cx| {
                                     let mut highlights = Vec::new();
                                     let background = cx.background_executor();
-                                    let mut rng = background.rng();
+                                    let rng = background.rng();
+                                    let mut rng = rng.lock();
 
                                     let highlight_count = rng.random_range(1..=5);
                                     for _ in 0..highlight_count {
