@@ -1,4 +1,4 @@
-use lsp::VersionedTextDocumentIdentifier;
+use lsp::{Uri, VersionedTextDocumentIdentifier};
 use serde::{Deserialize, Serialize};
 
 pub enum CheckStatus {}
@@ -15,37 +15,22 @@ impl lsp::request::Request for CheckStatus {
     const METHOD: &'static str = "checkStatus";
 }
 
-pub enum SignInInitiate {}
+pub enum SignIn {}
 
 #[derive(Debug, Serialize, Deserialize)]
-pub struct SignInInitiateParams {}
+pub struct SignInParams {}
 
-#[derive(Debug, Serialize, Deserialize)]
-#[serde(tag = "status")]
-pub enum SignInInitiateResult {
-    AlreadySignedIn { user: String },
-    PromptUserDeviceFlow(PromptUserDeviceFlow),
-}
-
-#[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct PromptUserDeviceFlow {
     pub user_code: String,
-    pub verification_uri: String,
+    pub command: lsp::Command,
 }
 
-impl lsp::request::Request for SignInInitiate {
-    type Params = SignInInitiateParams;
-    type Result = SignInInitiateResult;
-    const METHOD: &'static str = "signInInitiate";
-}
-
-pub enum SignInConfirm {}
-
-#[derive(Debug, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub struct SignInConfirmParams {
-    pub user_code: String,
+impl lsp::request::Request for SignIn {
+    type Params = SignInParams;
+    type Result = PromptUserDeviceFlow;
+    const METHOD: &'static str = "signIn";
 }
 
 #[derive(Debug, Serialize, Deserialize)]
@@ -67,12 +52,6 @@ pub enum SignInStatus {
     NotSignedIn,
 }
 
-impl lsp::request::Request for SignInConfirm {
-    type Params = SignInConfirmParams;
-    type Result = SignInStatus;
-    const METHOD: &'static str = "signInConfirm";
-}
-
 pub enum SignOut {}
 
 #[derive(Debug, Serialize, Deserialize)]
@@ -89,17 +68,26 @@ impl lsp::request::Request for SignOut {
     const METHOD: &'static str = "signOut";
 }
 
-pub enum StatusNotification {}
+pub enum DidChangeStatus {}
 
 #[derive(Debug, Serialize, Deserialize)]
-pub struct StatusNotificationParams {
-    pub message: String,
-    pub status: String, // One of Normal/InProgress
+pub struct DidChangeStatusParams {
+    #[serde(default)]
+    pub message: Option<String>,
+    pub kind: StatusKind,
 }
 
-impl lsp::notification::Notification for StatusNotification {
-    type Params = StatusNotificationParams;
-    const METHOD: &'static str = "statusNotification";
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
+pub enum StatusKind {
+    Normal,
+    Error,
+    Warning,
+    Inactive,
+}
+
+impl lsp::notification::Notification for DidChangeStatus {
+    type Params = DidChangeStatusParams;
+    const METHOD: &'static str = "didChangeStatus";
 }
 
 pub enum SetEditorInfo {}
@@ -191,3 +179,121 @@ impl lsp::request::Request for NextEditSuggestions {
 
     const METHOD: &'static str = "textDocument/copilotInlineEdit";
 }
+
+pub(crate) struct DidFocus;
+
+#[derive(Serialize, Deserialize)]
+pub(crate) struct DidFocusParams {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub(crate) uri: Option<Uri>,
+}
+
+impl lsp::notification::Notification for DidFocus {
+    type Params = DidFocusParams;
+    const METHOD: &'static str = "textDocument/didFocus";
+}
+
+pub(crate) struct DidShowInlineEdit;
+
+#[derive(Serialize, Deserialize)]
+pub(crate) struct DidShowInlineEditParams {
+    pub(crate) item: serde_json::Value,
+}
+
+impl lsp::notification::Notification for DidShowInlineEdit {
+    type Params = DidShowInlineEditParams;
+    const METHOD: &'static str = "textDocument/didShowInlineEdit";
+}
+
+// Inline Completions (non-NES) - textDocument/inlineCompletion
+
+pub enum InlineCompletions {}
+
+#[derive(Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct InlineCompletionsParams {
+    pub text_document: VersionedTextDocumentIdentifier,
+    pub position: lsp::Position,
+    pub context: InlineCompletionContext,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub formatting_options: Option<FormattingOptions>,
+}
+
+#[derive(Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct InlineCompletionContext {
+    pub trigger_kind: InlineCompletionTriggerKind,
+}
+
+#[derive(Debug, Clone, Copy)]
+pub enum InlineCompletionTriggerKind {
+    Invoked = 1,
+    Automatic = 2,
+}
+
+impl Serialize for InlineCompletionTriggerKind {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        serializer.serialize_u8(*self as u8)
+    }
+}
+
+impl<'de> Deserialize<'de> for InlineCompletionTriggerKind {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        let value = u8::deserialize(deserializer)?;
+        match value {
+            1 => Ok(InlineCompletionTriggerKind::Invoked),
+            2 => Ok(InlineCompletionTriggerKind::Automatic),
+            _ => Err(serde::de::Error::custom("invalid trigger kind")),
+        }
+    }
+}
+
+#[derive(Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct FormattingOptions {
+    pub tab_size: u32,
+    pub insert_spaces: bool,
+}
+
+#[derive(Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct InlineCompletionsResult {
+    pub items: Vec<InlineCompletionItem>,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct InlineCompletionItem {
+    pub insert_text: String,
+    pub range: lsp::Range,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub command: Option<lsp::Command>,
+}
+
+impl lsp::request::Request for InlineCompletions {
+    type Params = InlineCompletionsParams;
+    type Result = InlineCompletionsResult;
+
+    const METHOD: &'static str = "textDocument/inlineCompletion";
+}
+
+// Telemetry notifications for inline completions
+
+pub(crate) struct DidShowCompletion;
+
+#[derive(Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub(crate) struct DidShowCompletionParams {
+    pub(crate) item: InlineCompletionItem,
+}
+
+impl lsp::notification::Notification for DidShowCompletion {
+    type Params = DidShowCompletionParams;
+    const METHOD: &'static str = "textDocument/didShowCompletion";
+}
