@@ -1,10 +1,11 @@
 use crate::{BufferDiagnosticsEditor, ProjectDiagnosticsEditor, ToggleDiagnosticsRefresh};
 use gpui::{Context, EventEmitter, ParentElement, Render, Window};
 use language::DiagnosticEntry;
+use search::buffer_search;
 use text::{Anchor, BufferId};
-use ui::prelude::*;
-use ui::{IconButton, IconButtonShape, IconName, Tooltip};
+use ui::{Tooltip, prelude::*};
 use workspace::{ToolbarItemEvent, ToolbarItemLocation, ToolbarItemView, item::ItemHandle};
+use zed_actions::assistant::InlineAssist;
 
 pub struct ToolbarControls {
     editor: Option<Box<dyn DiagnosticsToolbarEditor>>,
@@ -45,28 +46,44 @@ impl Render for ToolbarControls {
             None => {}
         }
 
-        let warning_tooltip = if include_warnings {
-            "Exclude Warnings"
+        let (warning_tooltip, warning_color) = if include_warnings {
+            ("Exclude Warnings", Color::Warning)
         } else {
-            "Include Warnings"
-        };
-
-        let warning_color = if include_warnings {
-            Color::Warning
-        } else {
-            Color::Muted
+            ("Include Warnings", Color::Disabled)
         };
 
         h_flex()
             .gap_1()
+            .child({
+                IconButton::new("toggle_search", IconName::MagnifyingGlass)
+                    .icon_size(IconSize::Small)
+                    .tooltip(Tooltip::for_action_title(
+                        "Buffer Search",
+                        &buffer_search::Deploy::find(),
+                    ))
+                    .on_click(|_, window, cx| {
+                        window.dispatch_action(Box::new(buffer_search::Deploy::find()), cx);
+                    })
+            })
+            .child({
+                IconButton::new("inline_assist", IconName::ZedAssistant)
+                    .icon_size(IconSize::Small)
+                    .tooltip(Tooltip::for_action_title(
+                        "Inline Assist",
+                        &InlineAssist::default(),
+                    ))
+                    .on_click(|_, window, cx| {
+                        window.dispatch_action(Box::new(InlineAssist::default()), cx);
+                    })
+            })
             .map(|div| {
                 if is_updating {
                     div.child(
                         IconButton::new("stop-updating", IconName::Stop)
-                            .icon_color(Color::Info)
-                            .shape(IconButtonShape::Square)
+                            .icon_color(Color::Error)
+                            .icon_size(IconSize::Small)
                             .tooltip(Tooltip::for_action_title(
-                                "Stop diagnostics update",
+                                "Stop Siagnostics Update",
                                 &ToggleDiagnosticsRefresh,
                             ))
                             .on_click(cx.listener(move |toolbar_controls, _, _, cx| {
@@ -79,10 +96,9 @@ impl Render for ToolbarControls {
                 } else {
                     div.child(
                         IconButton::new("refresh-diagnostics", IconName::ArrowCircle)
-                            .icon_color(Color::Info)
-                            .shape(IconButtonShape::Square)
+                            .icon_size(IconSize::Small)
                             .tooltip(Tooltip::for_action_title(
-                                "Refresh diagnostics",
+                                "Refresh Diagnostics",
                                 &ToggleDiagnosticsRefresh,
                             ))
                             .on_click(cx.listener({
@@ -98,7 +114,7 @@ impl Render for ToolbarControls {
             .child(
                 IconButton::new("toggle-warnings", IconName::Warning)
                     .icon_color(warning_color)
-                    .shape(IconButtonShape::Square)
+                    .icon_size(IconSize::Small)
                     .tooltip(Tooltip::text(warning_tooltip))
                     .on_click(cx.listener(|this, _, window, cx| {
                         if let Some(editor) = &this.editor {
