@@ -9,16 +9,19 @@ use feature_flags::FeatureFlagAppExt as _;
 use gpui::{App, Entity, Task};
 use language::{Buffer, ToPoint as _};
 use project::{Project, WorktreeId};
-use std::{collections::hash_map, fmt::Write as _, path::Path, sync::Arc};
-use text::BufferSnapshot as TextBufferSnapshot;
+use std::{collections::hash_map, fmt::Write as _, ops::Range, path::Path, sync::Arc};
+use text::{BufferSnapshot as TextBufferSnapshot, Point};
 
 pub(crate) const DEFAULT_EXAMPLE_CAPTURE_RATE_PER_10K_PREDICTIONS: u16 = 10;
+pub(crate) const DEFAULT_STAFF_EXAMPLE_CAPTURE_RATE_PER_10K_PREDICTIONS: u16 = 100;
+pub(crate) const ZETA2_TESTING_RATE_PER_10K_PREDICTION: u16 = 500;
 
 pub fn capture_example(
     project: Entity<Project>,
     buffer: Entity<Buffer>,
     cursor_anchor: language::Anchor,
     mut events: Vec<StoredEvent>,
+    populate_expected_patch: bool,
     cx: &mut App,
 ) -> Option<Task<Result<ExampleSpec>>> {
     let snapshot = buffer.read(cx).snapshot();
@@ -56,7 +59,7 @@ pub fn capture_example(
             .and_then(|lang| lang.config().line_comments.first())
             .map(|s| s.to_string())
             .unwrap_or_default();
-        let (cursor_excerpt, cursor_offset) = cx
+        let (cursor_excerpt, cursor_offset, cursor_excerpt_range) = cx
             .background_executor()
             .spawn(async move { compute_cursor_excerpt(&snapshot, cursor_anchor) })
             .await;
@@ -73,6 +76,30 @@ pub fn capture_example(
             }
         }
 
+        // Initialize an empty patch with context lines, to make it easy
+        // to write the expected patch by hand.
+        let mut expected_patches = Vec::new();
+        let mut rejected_patch = None;
+        if populate_expected_patch {
+            let mut empty_patch = String::new();
+            let start_row = cursor_excerpt_range.start.row + 1;
+            let row_count = cursor_excerpt_range.end.row - cursor_excerpt_range.start.row + 1;
+            writeln!(&mut empty_patch, "--- a/{}", cursor_path.display()).ok();
+            writeln!(&mut empty_patch, "+++ b/{}", cursor_path.display()).ok();
+            writeln!(
+                &mut empty_patch,
+                "@@ -{},{} +{},{} @@",
+                start_row, row_count, start_row, row_count,
+            )
+            .ok();
+            for line in cursor_excerpt.lines() {
+                writeln!(&mut empty_patch, " {}", line).ok();
+            }
+
+            expected_patches.push(empty_patch.clone());
+            rejected_patch = Some(empty_patch);
+        }
+
         let mut spec = ExampleSpec {
             name: generate_timestamp_name(),
             repository_url,
@@ -83,7 +110,8 @@ pub fn capture_example(
             cursor_path,
             cursor_position: String::new(),
             edit_history,
-            expected_patches: Vec::new(),
+            expected_patches,
+            rejected_patch,
         };
         spec.set_cursor_excerpt(&cursor_excerpt, cursor_offset, &line_comment_prefix);
         Ok(spec)
@@ -124,7 +152,7 @@ fn write_event_with_relative_paths(
 fn compute_cursor_excerpt(
     snapshot: &language::BufferSnapshot,
     cursor_anchor: language::Anchor,
-) -> (String, usize) {
+) -> (String, usize, Range<Point>) {
     use text::ToOffset as _;
 
     let cursor_point = cursor_anchor.to_point(snapshot);
@@ -133,8 +161,10 @@ fn compute_cursor_excerpt(
     let context_start_offset = context_range.start.to_offset(snapshot);
     let cursor_offset = cursor_anchor.to_offset(snapshot);
     let cursor_offset_in_excerpt = cursor_offset.saturating_sub(context_start_offset);
-    let excerpt = snapshot.text_for_range(context_range).collect::<String>();
-    (excerpt, cursor_offset_in_excerpt)
+    let excerpt = snapshot
+        .text_for_range(context_range.clone())
+        .collect::<String>();
+    (excerpt, cursor_offset_in_excerpt, context_range)
 }
 
 async fn collect_snapshots(
@@ -153,19 +183,19 @@ async fn collect_snapshots(
                 .filter(|path| path.worktree_id == worktree_id)?;
             let relative_path: Arc<Path> = project_path.path.as_std_path().into();
             Some((project_path, relative_path))
-        })? {
+        }) {
             if let hash_map::Entry::Vacant(entry) = snapshots_by_path.entry(relative_path) {
                 let buffer = project
                     .update(cx, |project, cx| {
                         project.open_buffer(project_path.clone(), cx)
-                    })?
+                    })
                     .await?;
                 let diff = git_store
                     .update(cx, |git_store, cx| {
                         git_store.open_uncommitted_diff(buffer.clone(), cx)
-                    })?
+                    })
                     .await?;
-                let diff_snapshot = diff.update(cx, |diff, cx| diff.snapshot(cx))?;
+                let diff_snapshot = diff.update(cx, |diff, cx| diff.snapshot(cx));
                 entry.insert((stored_event.old_snapshot.clone(), diff_snapshot));
             }
         }
@@ -208,14 +238,23 @@ fn generate_timestamp_name() -> String {
 }
 
 pub(crate) fn should_sample_edit_prediction_example_capture(cx: &App) -> bool {
+    let default_rate = if cx.is_staff() {
+        DEFAULT_STAFF_EXAMPLE_CAPTURE_RATE_PER_10K_PREDICTIONS
+    } else {
+        DEFAULT_EXAMPLE_CAPTURE_RATE_PER_10K_PREDICTIONS
+    };
     let capture_rate = language::language_settings::all_language_settings(None, cx)
         .edit_predictions
         .example_capture_rate
-        .unwrap_or(DEFAULT_EXAMPLE_CAPTURE_RATE_PER_10K_PREDICTIONS);
+        .unwrap_or(default_rate);
     cx.has_flag::<EditPredictionExampleCaptureFeatureFlag>()
         && rand::random::<u16>() % 10_000 < capture_rate
 }
 
+pub(crate) fn should_send_testing_zeta2_request() -> bool {
+    rand::random::<u16>() % 10_000 < ZETA2_TESTING_RATE_PER_10K_PREDICTION
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -375,7 +414,15 @@ mod tests {
 
         let mut example = cx
             .update(|cx| {
-                capture_example(project.clone(), buffer.clone(), Anchor::MIN, events, cx).unwrap()
+                capture_example(
+                    project.clone(),
+                    buffer.clone(),
+                    Anchor::MIN,
+                    events,
+                    true,
+                    cx,
+                )
+                .unwrap()
             })
             .await
             .unwrap();
@@ -443,7 +490,52 @@ mod tests {
                          seven();
                 "}
                 .to_string(),
-                expected_patches: Vec::new()
+                expected_patches: vec![
+                    indoc! {"
+                        --- a/src/main.rs
+                        +++ b/src/main.rs
+                        @@ -1,16 +1,16 @@
+                         fn main() {
+                             // comment 1
+                             one();
+                             two();
+                             // comment 4
+                             three();
+                             four();
+                             // comment 3
+                             five();
+                             six();
+                             seven();
+                             eight();
+                             // comment 2
+                             nine();
+                         }
+                    "}
+                    .to_string()
+                ],
+                rejected_patch: Some(
+                    indoc! {"
+                        --- a/src/main.rs
+                        +++ b/src/main.rs
+                        @@ -1,16 +1,16 @@
+                         fn main() {
+                             // comment 1
+                             one();
+                             two();
+                             // comment 4
+                             three();
+                             four();
+                             // comment 3
+                             five();
+                             six();
+                             seven();
+                             eight();
+                             // comment 2
+                             nine();
+                         }
+                    "}
+                    .to_string()
+                )
             }
         );
     }
