@@ -6,7 +6,7 @@ use crate::example::Example;
 pub async fn run_distill(example: &mut Example) -> Result<()> {
     let predictions = mem::take(&mut example.predictions)
         .into_iter()
-        .map(|p| p.actual_patch)
+        .filter_map(|p| p.actual_patch)
         .collect();
 
     example.spec.expected_patches = predictions;
