@@ -155,7 +155,11 @@ impl ToString for Patch {
             } else {
                 result.push_str(&format!("--- a/{}\n", current_file));
             }
-            result.push_str(&format!("+++ b/{}\n", current_file));
+            if hunk.is_file_deletion() {
+                result.push_str("+++ /dev/null\n");
+            } else {
+                result.push_str(&format!("+++ b/{}\n", current_file));
+            }
             result.push_str(&hunk.to_string());
         }
 
@@ -190,10 +194,16 @@ impl Patch {
                 is_filename_inherited = true;
             } else if let Some(path) = line.strip_prefix("--- ") {
                 is_filename_inherited = false;
-                current_file = path.trim().strip_prefix("a/").unwrap_or(path).into();
+                let path = path.trim().strip_prefix("a/").unwrap_or(path);
+                if path != "/dev/null" {
+                    current_file = path.into();
+                }
             } else if let Some(path) = line.strip_prefix("+++ ") {
                 is_filename_inherited = false;
-                current_file = path.trim().strip_prefix("b/").unwrap_or(path).into();
+                let path = path.trim().strip_prefix("b/").unwrap_or(path);
+                if path != "/dev/null" {
+                    current_file = path.into();
+                }
             } else if let Some(line) = line.strip_prefix("+") {
                 hunk.lines.push(PatchLine::Addition(line.to_string()));
             } else if let Some(line) = line.strip_prefix("-") {
@@ -339,6 +349,11 @@ impl Hunk {
         self.old_start == 0 && self.old_count == 0
     }
 
+    /// Returns true if this hunk represents a file deletion (new side is empty).
+    pub fn is_file_deletion(&self) -> bool {
+        self.new_start == 0 && self.new_count == 0
+    }
+
     /// Render the hunk header
     pub fn header_string(&self) -> String {
         format!(
@@ -1495,4 +1510,31 @@ mod tests {
         "}
         );
     }
+
+    #[test]
+    fn test_file_deletion_diff_header() {
+        // When new_start and new_count are both 0, the file is being deleted,
+        // so the +++ line should be /dev/null instead of b/filename
+        let patch = Patch::parse_unified_diff(indoc! {"
+            --- a/old_file.rs
+            +++ /dev/null
+            @@ -1,3 +0,0 @@
+            -fn main() {
+            -    println!(\"goodbye\");
+            -}
+        "});
+
+        let actual = patch.to_string();
+        assert_eq!(
+            actual,
+            indoc! {"
+            --- a/old_file.rs
+            +++ /dev/null
+            @@ -1,3 +0,0 @@
+            -fn main() {
+            -    println!(\"goodbye\");
+            -}
+        "}
+        );
+    }
 }
