@@ -1,8 +1,8 @@
 use crate::{
-    example::{Example, ExampleContext},
+    example::Example,
     headless::EpAppState,
     load_project::run_load_project,
-    progress::{InfoStyle, Progress, Step, StepProgress},
+    progress::{ExampleProgress, InfoStyle, Step, StepProgress},
 };
 use anyhow::Context as _;
 use collections::HashSet;
@@ -17,36 +17,38 @@ use std::time::Duration;
 pub async fn run_context_retrieval(
     example: &mut Example,
     app_state: Arc<EpAppState>,
+    example_progress: &ExampleProgress,
     mut cx: AsyncApp,
 ) -> anyhow::Result<()> {
-    if example.context.is_some() {
+    if example
+        .prompt_inputs
+        .as_ref()
+        .is_some_and(|inputs| inputs.related_files.is_some())
+    {
         return Ok(());
     }
 
-    run_load_project(example, app_state.clone(), cx.clone()).await?;
+    run_load_project(example, app_state.clone(), example_progress, cx.clone()).await?;
 
-    let step_progress: Arc<StepProgress> = Progress::global()
-        .start(Step::Context, &example.spec.name)
-        .into();
+    let step_progress: Arc<StepProgress> = example_progress.start(Step::Context).into();
 
     let state = example.state.as_ref().unwrap();
     let project = state.project.clone();
 
     let _lsp_handle = project.update(&mut cx, |project, cx| {
         project.register_buffer_with_language_servers(&state.buffer, cx)
-    })?;
+    });
     wait_for_language_servers_to_start(&project, &state.buffer, &step_progress, &mut cx).await?;
 
-    let ep_store = cx.update(|cx| {
-        EditPredictionStore::try_global(cx).context("EditPredictionStore not initialized")
-    })??;
+    let ep_store = cx
+        .update(|cx| EditPredictionStore::try_global(cx))
+        .context("EditPredictionStore not initialized")?;
 
     let mut events = ep_store.update(&mut cx, |store, cx| {
         store.register_buffer(&state.buffer, &project, cx);
-        store.set_use_context(true);
         store.refresh_context(&project, &state.buffer, state.cursor_position, cx);
         store.debug_info(&project, cx)
-    })?;
+    });
 
     while let Some(event) = events.next().await {
         match event {
@@ -58,14 +60,14 @@ pub async fn run_context_retrieval(
     }
 
     let context_files =
-        ep_store.update(&mut cx, |store, cx| store.context_for_project(&project, cx))?;
+        ep_store.update(&mut cx, |store, cx| store.context_for_project(&project, cx));
 
     let excerpt_count: usize = context_files.iter().map(|f| f.excerpts.len()).sum();
     step_progress.set_info(format!("{} excerpts", excerpt_count), InfoStyle::Normal);
 
-    example.context = Some(ExampleContext {
-        files: context_files,
-    });
+    if let Some(prompt_inputs) = example.prompt_inputs.as_mut() {
+        prompt_inputs.related_files = Some(context_files);
+    }
     Ok(())
 }
 
@@ -75,10 +77,10 @@ async fn wait_for_language_servers_to_start(
     step_progress: &Arc<StepProgress>,
     cx: &mut AsyncApp,
 ) -> anyhow::Result<()> {
-    let lsp_store = project.read_with(cx, |project, _| project.lsp_store())?;
+    let lsp_store = project.read_with(cx, |project, _| project.lsp_store());
 
-    let (language_server_ids, mut starting_language_server_ids) = buffer
-        .update(cx, |buffer, cx| {
+    let (language_server_ids, mut starting_language_server_ids) =
+        buffer.update(cx, |buffer, cx| {
             lsp_store.update(cx, |lsp_store, cx| {
                 let ids = lsp_store.language_servers_for_local_buffer(buffer, cx);
                 let starting_ids = ids
@@ -88,15 +90,17 @@ async fn wait_for_language_servers_to_start(
                     .collect::<HashSet<_>>();
                 (ids, starting_ids)
             })
-        })
-        .unwrap_or_default();
+        });
 
     step_progress.set_substatus(format!("waiting for {} LSPs", language_server_ids.len()));
 
-    let timeout = cx
-        .background_executor()
-        .timer(Duration::from_secs(60 * 5))
-        .shared();
+    let timeout_duration = if starting_language_server_ids.is_empty() {
+        Duration::from_secs(30)
+    } else {
+        Duration::from_secs(60 * 5)
+    };
+
+    let timeout = cx.background_executor().timer(timeout_duration).shared();
 
     let (mut tx, mut rx) = mpsc::channel(language_server_ids.len());
     let added_subscription = cx.subscribe(project, {
@@ -118,7 +122,7 @@ async fn wait_for_language_servers_to_start(
                 }
             },
             _ = timeout.clone().fuse() => {
-                return Err(anyhow::anyhow!("LSP wait timed out after 5 minutes"));
+                return Err(anyhow::anyhow!("LSP wait timed out after {} minutes", timeout_duration.as_secs() / 60));
             }
         }
     }
@@ -164,7 +168,7 @@ async fn wait_for_language_servers_to_start(
     ];
 
     project
-        .update(cx, |project, cx| project.save_buffer(buffer.clone(), cx))?
+        .update(cx, |project, cx| project.save_buffer(buffer.clone(), cx))
         .await?;
 
     let mut pending_language_server_ids = lsp_store.read_with(cx, |lsp_store, _| {
@@ -178,7 +182,7 @@ async fn wait_for_language_servers_to_start(
                     .is_some_and(|status| status.has_pending_diagnostic_updates)
             })
             .collect::<HashSet<_>>()
-    })?;
+    });
     while !pending_language_server_ids.is_empty() {
         futures::select! {
             language_server_id = rx.next() => {
@@ -187,7 +191,7 @@ async fn wait_for_language_servers_to_start(
                 }
             },
             _ = timeout.clone().fuse() => {
-                return Err(anyhow::anyhow!("LSP wait timed out after 5 minutes"));
+                return Err(anyhow::anyhow!("LSP wait timed out after {} minutes", timeout_duration.as_secs() / 60));
             }
         }
     }
