@@ -153,11 +153,9 @@ impl EditPredictionContextView {
         run.finished_at = Some(info.timestamp);
         run.metadata = info.metadata;
 
-        let related_files = self
-            .store
-            .read(cx)
-            .context_for_project_with_buffers(&self.project, cx)
-            .map_or(Vec::new(), |files| files.collect());
+        let related_files = self.store.update(cx, |store, cx| {
+            store.context_for_project_with_buffers(&self.project, cx)
+        });
 
         let editor = run.editor.clone();
         let multibuffer = run.editor.read(cx).buffer().clone();
@@ -188,7 +186,7 @@ impl EditPredictionContextView {
                 for (path, buffer, ranges) in paths {
                     multibuffer.set_excerpts_for_path(path, buffer, ranges, 0, cx);
                 }
-            })?;
+            });
 
             editor.update_in(cx, |editor, window, cx| {
                 editor.move_to_beginning(&Default::default(), window, cx);
@@ -243,11 +241,14 @@ impl EditPredictionContextView {
             .gap_2()
             .child(v_flex().h_full().flex_1().child({
                 let t0 = run.started_at;
-                let mut table = ui::Table::<2>::new().width(ui::px(300.)).no_ui_font();
+                let mut table = ui::Table::new(2).width(ui::px(300.)).no_ui_font();
                 for (key, value) in &run.metadata {
-                    table = table.row([key.into_any_element(), value.clone().into_any_element()])
+                    table = table.row(vec![
+                        key.into_any_element(),
+                        value.clone().into_any_element(),
+                    ])
                 }
-                table = table.row([
+                table = table.row(vec![
                     "Total Time".into_any_element(),
                     format!("{} ms", (run.finished_at.unwrap_or(t0) - t0).as_millis())
                         .into_any_element(),
