@@ -45,10 +45,10 @@ pub fn switch_source_header(
                 .map(|file| file.path())
                 .map(|path| path.display(PathStyle::local()).to_string())
                 .unwrap_or_else(|| "Unknown".to_string())
-        })?;
+        });
 
         let switch_source_header = if let Some((client, project_id)) = upstream_client {
-            let buffer_id = buffer.read_with(cx, |buffer, _| buffer.remote_id())?;
+            let buffer_id = buffer.read_with(cx, |buffer, _| buffer.remote_id());
             let request = proto::LspExtSwitchSourceHeader {
                 project_id,
                 buffer_id: buffer_id.to_proto(),
@@ -67,7 +67,7 @@ pub fn switch_source_header(
                         project::lsp_store::lsp_ext_command::SwitchSourceHeader,
                         cx,
                     )
-                })?
+                })
                 .await
                 .with_context(|| {
                     format!("Switch source/header LSP request for path \"{source_file}\" failed")
