@@ -5,7 +5,7 @@ use crate::{
 };
 use gpui::{Bounds, Context, Pixels, Window};
 use language::Point;
-use multi_buffer::{Anchor, ToPoint};
+use multi_buffer::Anchor;
 use std::cmp;
 
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
@@ -186,18 +186,7 @@ impl Editor {
             }
         }
 
-        let style = self.style(cx).clone();
-        let sticky_headers = self.sticky_headers(&style, cx).unwrap_or_default();
-        let visible_sticky_headers = sticky_headers
-            .iter()
-            .filter(|h| {
-                let buffer_snapshot = display_map.buffer_snapshot();
-                let buffer_range =
-                    h.range.start.to_point(buffer_snapshot)..h.range.end.to_point(buffer_snapshot);
-
-                buffer_range.contains(&Point::new(target_top as u32, 0))
-            })
-            .count();
+        let visible_sticky_headers = self.scroll_manager.sticky_header_line_count();
 
         let margin = if matches!(self.mode, EditorMode::AutoHeight { .. }) {
             0.
@@ -207,21 +196,17 @@ impl Editor {
 
         let strategy = match autoscroll {
             Autoscroll::Strategy(strategy, _) => strategy,
-            Autoscroll::Next => {
-                let last_autoscroll = &self.scroll_manager.last_autoscroll;
-                if let Some(last_autoscroll) = last_autoscroll {
-                    if self.scroll_manager.anchor.offset == last_autoscroll.0
-                        && target_top == last_autoscroll.1
-                        && target_bottom == last_autoscroll.2
-                    {
-                        last_autoscroll.3.next()
-                    } else {
-                        AutoscrollStrategy::default()
-                    }
-                } else {
-                    AutoscrollStrategy::default()
-                }
-            }
+            Autoscroll::Next => self
+                .scroll_manager
+                .last_autoscroll
+                .as_ref()
+                .filter(|(offset, last_target_top, last_target_bottom, _)| {
+                    self.scroll_manager.anchor.offset == *offset
+                        && target_top == *last_target_top
+                        && target_bottom == *last_target_bottom
+                })
+                .map(|(_, _, _, strategy)| strategy.next())
+                .unwrap_or_default(),
         };
         if let Autoscroll::Strategy(_, Some(anchor)) = autoscroll {
             target_top = anchor.to_display_point(&display_map).row().as_f64();
