@@ -7,8 +7,8 @@ use async_compression::futures::bufread::GzipEncoder;
 use collections::{BTreeMap, HashSet};
 use extension::ExtensionHostProxy;
 use fs::{FakeFs, Fs, RealFs};
-use futures::{AsyncReadExt, StreamExt, io::BufReader};
-use gpui::{AppContext as _, TestAppContext};
+use futures::{AsyncReadExt, FutureExt, StreamExt, io::BufReader};
+use gpui::{AppContext as _, BackgroundExecutor, TestAppContext};
 use http_client::{FakeHttpClient, Response};
 use language::{BinaryStatus, LanguageMatcher, LanguageName, LanguageRegistry};
 use language_extension::LspAccess;
@@ -534,10 +534,26 @@ async fn test_extension_store(cx: &mut TestAppContext) {
 
 #[gpui::test]
 async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
-    log::info!("Initializing test");
     init_test(cx);
     cx.executor().allow_parking();
 
+    let executor = cx.executor();
+    async fn await_or_timeout<T>(
+        executor: &BackgroundExecutor,
+        what: &'static str,
+        seconds: u64,
+        future: impl std::future::Future<Output = T>,
+    ) -> T {
+        let timeout = executor.timer(std::time::Duration::from_secs(seconds));
+
+        futures::select! {
+            output = future.fuse() => output,
+            _ = futures::FutureExt::fuse(timeout) => panic!(
+            "[test_extension_store_with_test_extension] timed out after {seconds}s while {what}"
+        )
+        }
+    }
+
     let root_dir = Path::new(env!("CARGO_MANIFEST_DIR"))
         .parent()
         .unwrap()
@@ -559,9 +575,13 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
     let extensions_dir = extensions_tree.path().canonicalize().unwrap();
     let project_dir = project_dir.path().canonicalize().unwrap();
 
-    log::info!("Setting up test");
-
-    let project = Project::test(fs.clone(), [project_dir.as_path()], cx).await;
+    let project = await_or_timeout(
+        &executor,
+        "awaiting Project::test",
+        5,
+        Project::test(fs.clone(), [project_dir.as_path()], cx),
+    )
+    .await;
 
     let proxy = Arc::new(ExtensionHostProxy::new());
     let theme_registry = Arc::new(ThemeRegistry::new(Box::new(())));
@@ -679,8 +699,6 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
         )
     });
 
-    log::info!("Flushing events");
-
     // Ensure that debounces fire.
     let mut events = cx.events(&extension_store);
     let executor = cx.executor();
@@ -701,12 +719,17 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
         .detach();
     });
 
-    extension_store
-        .update(cx, |store, cx| {
+    let executor = cx.executor();
+    await_or_timeout(
+        &executor,
+        "awaiting install_dev_extension",
+        60,
+        extension_store.update(cx, |store, cx| {
             store.install_dev_extension(test_extension_dir.clone(), cx)
-        })
-        .await
-        .unwrap();
+        }),
+    )
+    .await
+    .unwrap();
 
     let mut fake_servers = language_registry.register_fake_lsp_server(
         LanguageServerName("gleam".into()),
@@ -716,15 +739,29 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
         },
         None,
     );
+    cx.executor().run_until_parked();
 
-    let (buffer, _handle) = project
-        .update(cx, |project, cx| {
+    let (buffer, _handle) = await_or_timeout(
+        &executor,
+        "awaiting open_local_buffer_with_lsp",
+        5,
+        project.update(cx, |project, cx| {
             project.open_local_buffer_with_lsp(project_dir.join("test.gleam"), cx)
-        })
-        .await
-        .unwrap();
+        }),
+    )
+    .await
+    .unwrap();
+    cx.executor().run_until_parked();
+
+    let fake_server = await_or_timeout(
+        &executor,
+        "awaiting first fake server spawn",
+        10,
+        fake_servers.next(),
+    )
+    .await
+    .unwrap();
 
-    let fake_server = fake_servers.next().await.unwrap();
     let work_dir = extensions_dir.join(format!("work/{test_extension_id}"));
     let expected_server_path = work_dir.join("gleam-v1.2.3/gleam");
     let expected_binary_contents = language_server_version.lock().binary_contents.clone();
@@ -738,16 +775,51 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
     assert_eq!(fake_server.binary.path, expected_server_path);
     assert_eq!(fake_server.binary.arguments, [OsString::from("lsp")]);
     assert_eq!(
-        fs.load(&expected_server_path).await.unwrap(),
+        await_or_timeout(
+            &executor,
+            "awaiting fs.load(expected_server_path)",
+            5,
+            fs.load(&expected_server_path)
+        )
+        .await
+        .unwrap(),
         expected_binary_contents
     );
     assert_eq!(language_server_version.lock().http_request_count, 2);
     assert_eq!(
         [
-            status_updates.next().await.unwrap(),
-            status_updates.next().await.unwrap(),
-            status_updates.next().await.unwrap(),
-            status_updates.next().await.unwrap(),
+            await_or_timeout(
+                &executor,
+                "awaiting status_updates #1",
+                5,
+                status_updates.next()
+            )
+            .await
+            .unwrap(),
+            await_or_timeout(
+                &executor,
+                "awaiting status_updates #2",
+                5,
+                status_updates.next()
+            )
+            .await
+            .unwrap(),
+            await_or_timeout(
+                &executor,
+                "awaiting status_updates #3",
+                5,
+                status_updates.next()
+            )
+            .await
+            .unwrap(),
+            await_or_timeout(
+                &executor,
+                "awaiting status_updates #4",
+                5,
+                status_updates.next()
+            )
+            .await
+            .unwrap(),
         ],
         [
             (
@@ -796,16 +868,36 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
         ])))
     });
 
-    let completion_labels = project
-        .update(cx, |project, cx| {
+    // `register_fake_lsp_server` can yield a server instance before the client has finished the LSP
+    // initialization handshake. Wait until we observe the client's `initialized` notification before
+    // issuing requests like completion.
+    await_or_timeout(
+        &executor,
+        "awaiting LSP Initialized notification",
+        5,
+        async {
+            fake_server
+                .clone()
+                .try_receive_notification::<lsp::notification::Initialized>()
+                .await;
+        },
+    )
+    .await;
+
+    let completion_labels = await_or_timeout(
+        &executor,
+        "awaiting completions",
+        5,
+        project.update(cx, |project, cx| {
             project.completions(&buffer, 0, DEFAULT_COMPLETION_CONTEXT, cx)
-        })
-        .await
-        .unwrap()
-        .into_iter()
-        .flat_map(|response| response.completions)
-        .map(|c| c.label.text)
-        .collect::<Vec<_>>();
+        }),
+    )
+    .await
+    .unwrap()
+    .into_iter()
+    .flat_map(|response| response.completions)
+    .map(|c| c.label.text)
+    .collect::<Vec<_>>();
     assert_eq!(
         completion_labels,
         [
@@ -829,40 +921,82 @@ async fn test_extension_store_with_test_extension(cx: &mut TestAppContext) {
 
     // The extension has cached the binary path, and does not attempt
     // to reinstall it.
-    let fake_server = fake_servers.next().await.unwrap();
+    let fake_server = await_or_timeout(
+        &executor,
+        "awaiting second fake server spawn",
+        5,
+        fake_servers.next(),
+    )
+    .await
+    .unwrap();
     assert_eq!(fake_server.binary.path, expected_server_path);
     assert_eq!(
-        fs.load(&expected_server_path).await.unwrap(),
+        await_or_timeout(
+            &executor,
+            "awaiting fs.load(expected_server_path) after restart",
+            5,
+            fs.load(&expected_server_path)
+        )
+        .await
+        .unwrap(),
         expected_binary_contents
     );
     assert_eq!(language_server_version.lock().http_request_count, 0);
 
     // Reload the extension, clearing its cache.
     // Start a new instance of the language server.
-    extension_store
-        .update(cx, |store, cx| {
+    await_or_timeout(
+        &executor,
+        "awaiting extension_store.reload(test-extension)",
+        5,
+        extension_store.update(cx, |store, cx| {
             store.reload(Some("test-extension".into()), cx)
-        })
-        .await;
+        }),
+    )
+    .await;
     cx.executor().run_until_parked();
     project.update(cx, |project, cx| {
         project.restart_language_servers_for_buffers(vec![buffer.clone()], HashSet::default(), cx)
     });
 
     // The extension re-fetches the latest version of the language server.
-    let fake_server = fake_servers.next().await.unwrap();
+    let fake_server = await_or_timeout(
+        &executor,
+        "awaiting third fake server spawn",
+        5,
+        fake_servers.next(),
+    )
+    .await
+    .unwrap();
     let new_expected_server_path =
         extensions_dir.join(format!("work/{test_extension_id}/gleam-v2.0.0/gleam"));
     let expected_binary_contents = language_server_version.lock().binary_contents.clone();
     assert_eq!(fake_server.binary.path, new_expected_server_path);
     assert_eq!(fake_server.binary.arguments, [OsString::from("lsp")]);
     assert_eq!(
-        fs.load(&new_expected_server_path).await.unwrap(),
+        await_or_timeout(
+            &executor,
+            "awaiting fs.load(new_expected_server_path)",
+            5,
+            fs.load(&new_expected_server_path)
+        )
+        .await
+        .unwrap(),
         expected_binary_contents
     );
 
     // The old language server directory has been cleaned up.
-    assert!(fs.metadata(&expected_server_path).await.unwrap().is_none());
+    assert!(
+        await_or_timeout(
+            &executor,
+            "awaiting fs.metadata(expected_server_path)",
+            5,
+            fs.metadata(&expected_server_path)
+        )
+        .await
+        .unwrap()
+        .is_none()
+    );
 }
 
 fn init_test(cx: &mut TestAppContext) {
