@@ -58,18 +58,22 @@ async fn run_git_blame(
     contents: &Rope,
     line_ending: LineEnding,
 ) -> Result<String> {
-    let mut child = util::command::new_smol_command(git_binary)
-        .current_dir(working_directory)
-        .arg("blame")
-        .arg("--incremental")
-        .arg("--contents")
-        .arg("-")
-        .arg(path.as_unix_str())
-        .stdin(Stdio::piped())
-        .stdout(Stdio::piped())
-        .stderr(Stdio::piped())
-        .spawn()
-        .context("starting git blame process")?;
+    let mut child = {
+        let span = ztracing::debug_span!("spawning git-blame command", path = path.as_unix_str());
+        let _enter = span.enter();
+        util::command::new_smol_command(git_binary)
+            .current_dir(working_directory)
+            .arg("blame")
+            .arg("--incremental")
+            .arg("--contents")
+            .arg("-")
+            .arg(path.as_unix_str())
+            .stdin(Stdio::piped())
+            .stdout(Stdio::piped())
+            .stderr(Stdio::piped())
+            .spawn()
+            .context("starting git blame process")?
+    };
 
     let stdin = child
         .stdin
