@@ -29,10 +29,6 @@ actions!(
     ]
 );
 
-pub fn register(workspace: &mut Workspace) {
-    workspace.register_action(open);
-}
-
 pub fn open(
     workspace: &mut Workspace,
     _: &zed_actions::git::ViewStash,
@@ -46,6 +42,16 @@ pub fn open(
     })
 }
 
+pub fn create_embedded(
+    repository: Option<Entity<Repository>>,
+    workspace: WeakEntity<Workspace>,
+    width: Rems,
+    window: &mut Window,
+    cx: &mut Context<StashList>,
+) -> StashList {
+    StashList::new_embedded(repository, workspace, width, window, cx)
+}
+
 pub struct StashList {
     width: Rems,
     pub picker: Entity<Picker<StashListDelegate>>,
@@ -60,6 +66,22 @@ impl StashList {
         width: Rems,
         window: &mut Window,
         cx: &mut Context<Self>,
+    ) -> Self {
+        let mut this = Self::new_inner(repository, workspace, width, false, window, cx);
+        this._subscriptions
+            .push(cx.subscribe(&this.picker, |_, _, _, cx| {
+                cx.emit(DismissEvent);
+            }));
+        this
+    }
+
+    fn new_inner(
+        repository: Option<Entity<Repository>>,
+        workspace: WeakEntity<Workspace>,
+        width: Rems,
+        embedded: bool,
+        window: &mut Window,
+        cx: &mut Context<Self>,
     ) -> Self {
         let mut _subscriptions = Vec::new();
         let stash_request = repository
@@ -103,16 +125,16 @@ impl StashList {
         .detach_and_log_err(cx);
 
         let delegate = StashListDelegate::new(repository, workspace, window, cx);
-        let picker = cx.new(|cx| Picker::uniform_list(delegate, window, cx));
+        let picker = cx.new(|cx| {
+            Picker::uniform_list(delegate, window, cx)
+                .show_scrollbar(true)
+                .modal(!embedded)
+        });
         let picker_focus_handle = picker.focus_handle(cx);
         picker.update(cx, |picker, _| {
             picker.delegate.focus_handle = picker_focus_handle.clone();
         });
 
-        _subscriptions.push(cx.subscribe(&picker, |_, _, _, cx| {
-            cx.emit(DismissEvent);
-        }));
-
         Self {
             picker,
             picker_focus_handle,
@@ -121,7 +143,22 @@ impl StashList {
         }
     }
 
-    fn handle_drop_stash(
+    fn new_embedded(
+        repository: Option<Entity<Repository>>,
+        workspace: WeakEntity<Workspace>,
+        width: Rems,
+        window: &mut Window,
+        cx: &mut Context<Self>,
+    ) -> Self {
+        let mut this = Self::new_inner(repository, workspace, width, true, window, cx);
+        this._subscriptions
+            .push(cx.subscribe(&this.picker, |_, _, _, cx| {
+                cx.emit(DismissEvent);
+            }));
+        this
+    }
+
+    pub fn handle_drop_stash(
         &mut self,
         _: &DropStashItem,
         window: &mut Window,
@@ -135,7 +172,7 @@ impl StashList {
         cx.notify();
     }
 
-    fn handle_show_stash(
+    pub fn handle_show_stash(
         &mut self,
         _: &ShowStashItem,
         window: &mut Window,
@@ -149,7 +186,7 @@ impl StashList {
         cx.notify();
     }
 
-    fn handle_modifiers_changed(
+    pub fn handle_modifiers_changed(
         &mut self,
         ev: &ModifiersChangedEvent,
         _: &mut Window,
@@ -246,7 +283,7 @@ impl StashListDelegate {
         };
 
         cx.spawn(async move |_, cx| {
-            repo.update(cx, |repo, cx| repo.stash_drop(Some(stash_index), cx))?
+            repo.update(cx, |repo, cx| repo.stash_drop(Some(stash_index), cx))
                 .await??;
             Ok(())
         })
@@ -281,7 +318,7 @@ impl StashListDelegate {
         };
 
         cx.spawn(async move |_, cx| {
-            repo.update(cx, |repo, cx| repo.stash_pop(Some(stash_index), cx))?
+            repo.update(cx, |repo, cx| repo.stash_pop(Some(stash_index), cx))
                 .await?;
             Ok(())
         })
@@ -297,7 +334,7 @@ impl StashListDelegate {
         };
 
         cx.spawn(async move |_, cx| {
-            repo.update(cx, |repo, cx| repo.stash_apply(Some(stash_index), cx))?
+            repo.update(cx, |repo, cx| repo.stash_apply(Some(stash_index), cx))
                 .await?;
             Ok(())
         })
