@@ -18,16 +18,12 @@ use project::{
 use recent_projects::{RemoteConnectionModal, connect};
 use remote::{RemoteConnectionOptions, remote_client::ConnectionIdentifier};
 use std::{path::PathBuf, sync::Arc};
-use ui::{HighlightedLabel, KeyBinding, ListItem, ListItemSpacing, Tooltip, prelude::*};
+use ui::{HighlightedLabel, KeyBinding, ListItem, ListItemSpacing, prelude::*};
 use util::ResultExt;
 use workspace::{ModalView, Workspace, notifications::DetachAndPromptErr};
 
 actions!(git, [WorktreeFromDefault, WorktreeFromDefaultOnWindow]);
 
-pub fn register(workspace: &mut Workspace) {
-    workspace.register_action(open);
-}
-
 pub fn open(
     workspace: &mut Workspace,
     _: &zed_actions::git::Worktree,
@@ -41,11 +37,22 @@ pub fn open(
     })
 }
 
+pub fn create_embedded(
+    repository: Option<Entity<Repository>>,
+    workspace: WeakEntity<Workspace>,
+    width: Rems,
+    window: &mut Window,
+    cx: &mut Context<WorktreeList>,
+) -> WorktreeList {
+    WorktreeList::new_embedded(repository, workspace, width, window, cx)
+}
+
 pub struct WorktreeList {
     width: Rems,
     pub picker: Entity<Picker<WorktreeListDelegate>>,
     picker_focus_handle: FocusHandle,
-    _subscription: Subscription,
+    _subscription: Option<Subscription>,
+    embedded: bool,
 }
 
 impl WorktreeList {
@@ -55,14 +62,29 @@ impl WorktreeList {
         width: Rems,
         window: &mut Window,
         cx: &mut Context<Self>,
+    ) -> Self {
+        let mut this = Self::new_inner(repository, workspace, width, false, window, cx);
+        this._subscription = Some(cx.subscribe(&this.picker, |_, _, _, cx| {
+            cx.emit(DismissEvent);
+        }));
+        this
+    }
+
+    fn new_inner(
+        repository: Option<Entity<Repository>>,
+        workspace: WeakEntity<Workspace>,
+        width: Rems,
+        embedded: bool,
+        window: &mut Window,
+        cx: &mut Context<Self>,
     ) -> Self {
         let all_worktrees_request = repository
             .clone()
             .map(|repository| repository.update(cx, |repository, _| repository.worktrees()));
 
-        let default_branch_request = repository
-            .clone()
-            .map(|repository| repository.update(cx, |repository, _| repository.default_branch()));
+        let default_branch_request = repository.clone().map(|repository| {
+            repository.update(cx, |repository, _| repository.default_branch(false))
+        });
 
         cx.spawn_in(window, async move |this, cx| {
             let all_worktrees = all_worktrees_request
@@ -90,25 +112,40 @@ impl WorktreeList {
         .detach_and_log_err(cx);
 
         let delegate = WorktreeListDelegate::new(workspace, repository, window, cx);
-        let picker = cx.new(|cx| Picker::uniform_list(delegate, window, cx));
+        let picker = cx.new(|cx| {
+            Picker::uniform_list(delegate, window, cx)
+                .show_scrollbar(true)
+                .modal(!embedded)
+        });
         let picker_focus_handle = picker.focus_handle(cx);
         picker.update(cx, |picker, _| {
             picker.delegate.focus_handle = picker_focus_handle.clone();
         });
 
-        let _subscription = cx.subscribe(&picker, |_, _, _, cx| {
-            cx.emit(DismissEvent);
-        });
-
         Self {
             picker,
             picker_focus_handle,
             width,
-            _subscription,
+            _subscription: None,
+            embedded,
         }
     }
 
-    fn handle_modifiers_changed(
+    fn new_embedded(
+        repository: Option<Entity<Repository>>,
+        workspace: WeakEntity<Workspace>,
+        width: Rems,
+        window: &mut Window,
+        cx: &mut Context<Self>,
+    ) -> Self {
+        let mut this = Self::new_inner(repository, workspace, width, true, window, cx);
+        this._subscription = Some(cx.subscribe(&this.picker, |_, _, _, cx| {
+            cx.emit(DismissEvent);
+        }));
+        this
+    }
+
+    pub fn handle_modifiers_changed(
         &mut self,
         ev: &ModifiersChangedEvent,
         _: &mut Window,
@@ -118,7 +155,7 @@ impl WorktreeList {
             .update(cx, |picker, _| picker.delegate.modifiers = ev.modifiers)
     }
 
-    fn handle_new_worktree(
+    pub fn handle_new_worktree(
         &mut self,
         replace_current_window: bool,
         window: &mut Window,
@@ -167,10 +204,12 @@ impl Render for WorktreeList {
                 this.handle_new_worktree(true, w, cx)
             }))
             .child(self.picker.clone())
-            .on_mouse_down_out({
-                cx.listener(move |this, _, window, cx| {
-                    this.picker.update(cx, |this, cx| {
-                        this.cancel(&Default::default(), window, cx);
+            .when(!self.embedded, |el| {
+                el.on_mouse_down_out({
+                    cx.listener(move |this, _, window, cx| {
+                        this.picker.update(cx, |this, cx| {
+                            this.cancel(&Default::default(), window, cx);
+                        })
                     })
                 })
             })
@@ -260,7 +299,7 @@ impl WorktreeListDelegate {
 
             repo.update(cx, |repo, _| {
                 repo.create_worktree(branch.clone(), path.clone(), commit)
-            })?
+            })
             .await??;
             let new_worktree_path = path.join(branch);
 
@@ -444,7 +483,7 @@ async fn open_remote_worktree(
         return Ok(());
     };
 
-    let new_project = cx.update(|cx| {
+    let new_project: Entity<project::Project> = cx.update(|cx| {
         project::Project::remote(
             session,
             app_state.client.clone(),
@@ -455,7 +494,7 @@ async fn open_remote_worktree(
             true,
             cx,
         )
-    })?;
+    });
 
     let window_to_use = if replace_current_window {
         workspace_window
@@ -463,12 +502,12 @@ async fn open_remote_worktree(
         let workspace_position = cx
             .update(|cx| {
                 workspace::remote_workspace_position_from_db(connection_options.clone(), &paths, cx)
-            })?
+            })
             .await
             .context("fetching workspace position from db")?;
 
         let mut options =
-            cx.update(|cx| (app_state.build_window_options)(workspace_position.display, cx))?;
+            cx.update(|cx| (app_state.build_window_options)(workspace_position.display, cx));
         options.window_bounds = workspace_position.window_bounds;
 
         cx.open_window(options, |window, cx| {
@@ -632,68 +671,32 @@ impl PickerDelegate for WorktreeListDelegate {
             .take(7)
             .collect::<String>();
 
-        let focus_handle = self.focus_handle.clone();
-        let icon = if let Some(default_branch) = self.default_branch.clone()
-            && entry.is_new
-        {
-            Some(
-                IconButton::new("worktree-from-default", IconName::GitBranchAlt)
-                    .on_click(|_, window, cx| {
-                        window.dispatch_action(WorktreeFromDefault.boxed_clone(), cx)
-                    })
-                    .on_right_click(|_, window, cx| {
-                        window.dispatch_action(WorktreeFromDefaultOnWindow.boxed_clone(), cx)
-                    })
-                    .tooltip(move |_, cx| {
-                        Tooltip::for_action_in(
-                            format!("From default branch {default_branch}"),
-                            &WorktreeFromDefault,
-                            &focus_handle,
-                            cx,
-                        )
-                    }),
+        let (branch_name, sublabel) = if entry.is_new {
+            (
+                Label::new(format!("Create Worktree: \"{}\"…", entry.worktree.branch()))
+                    .truncate()
+                    .into_any_element(),
+                format!(
+                    "based off {}",
+                    self.base_branch(cx).unwrap_or("the current branch")
+                ),
             )
         } else {
-            None
-        };
+            let branch = entry.worktree.branch();
+            let branch_first_line = branch.lines().next().unwrap_or(branch);
+            let positions: Vec<_> = entry
+                .positions
+                .iter()
+                .copied()
+                .filter(|&pos| pos < branch_first_line.len())
+                .collect();
 
-        let branch_name = if entry.is_new {
-            h_flex()
-                .gap_1()
-                .child(
-                    Icon::new(IconName::Plus)
-                        .size(IconSize::Small)
-                        .color(Color::Muted),
-                )
-                .child(
-                    Label::new(format!("Create worktree \"{}\"…", entry.worktree.branch()))
-                        .single_line()
-                        .truncate(),
-                )
-                .into_any_element()
-        } else {
-            h_flex()
-                .gap_1()
-                .child(
-                    Icon::new(IconName::GitBranch)
-                        .size(IconSize::Small)
-                        .color(Color::Muted),
-                )
-                .child(HighlightedLabel::new(
-                    entry.worktree.branch().to_owned(),
-                    entry.positions.clone(),
-                ))
-                .truncate()
-                .into_any_element()
-        };
-
-        let sublabel = if entry.is_new {
-            format!(
-                "based off {}",
-                self.base_branch(cx).unwrap_or("the current branch")
+            (
+                HighlightedLabel::new(branch_first_line.to_owned(), positions)
+                    .truncate()
+                    .into_any_element(),
+                path,
             )
-        } else {
-            format!("at {}", path)
         };
 
         Some(
@@ -704,33 +707,30 @@ impl PickerDelegate for WorktreeListDelegate {
                 .child(
                     v_flex()
                         .w_full()
-                        .overflow_hidden()
                         .child(
                             h_flex()
-                                .gap_6()
+                                .gap_2()
                                 .justify_between()
                                 .overflow_x_hidden()
                                 .child(branch_name)
-                                .when(!entry.is_new, |el| {
-                                    el.child(
+                                .when(!entry.is_new, |this| {
+                                    this.child(
                                         Label::new(sha)
                                             .size(LabelSize::Small)
                                             .color(Color::Muted)
+                                            .buffer_font(cx)
                                             .into_element(),
                                     )
                                 }),
                         )
                         .child(
-                            div().max_w_96().child(
-                                Label::new(sublabel)
-                                    .size(LabelSize::Small)
-                                    .color(Color::Muted)
-                                    .truncate()
-                                    .into_any_element(),
-                            ),
+                            Label::new(sublabel)
+                                .size(LabelSize::Small)
+                                .color(Color::Muted)
+                                .truncate()
+                                .into_any_element(),
                         ),
-                )
-                .end_slot::<IconButton>(icon),
+                ),
         )
     }
 
@@ -740,17 +740,42 @@ impl PickerDelegate for WorktreeListDelegate {
 
     fn render_footer(&self, _: &mut Window, cx: &mut Context<Picker<Self>>) -> Option<AnyElement> {
         let focus_handle = self.focus_handle.clone();
+        let selected_entry = self.matches.get(self.selected_index);
+        let is_creating = selected_entry.is_some_and(|entry| entry.is_new);
 
-        Some(
-            h_flex()
-                .w_full()
-                .p_1p5()
-                .gap_0p5()
-                .justify_end()
-                .border_t_1()
-                .border_color(cx.theme().colors().border_variant)
-                .child(
-                    Button::new("open-in-new-window", "Open in new window")
+        let footer_container = h_flex()
+            .w_full()
+            .p_1p5()
+            .gap_0p5()
+            .justify_end()
+            .border_t_1()
+            .border_color(cx.theme().colors().border_variant);
+
+        if is_creating {
+            let from_default_button = self.default_branch.as_ref().map(|default_branch| {
+                Button::new(
+                    "worktree-from-default",
+                    format!("Create from: {default_branch}"),
+                )
+                .key_binding(
+                    KeyBinding::for_action_in(&WorktreeFromDefault, &focus_handle, cx)
+                        .map(|kb| kb.size(rems_from_px(12.))),
+                )
+                .on_click(|_, window, cx| {
+                    window.dispatch_action(WorktreeFromDefault.boxed_clone(), cx)
+                })
+            });
+
+            let current_branch = self.base_branch(cx).unwrap_or("current branch");
+
+            Some(
+                footer_container
+                    .when_some(from_default_button, |this, button| this.child(button))
+                    .child(
+                        Button::new(
+                            "worktree-from-current",
+                            format!("Create from: {current_branch}"),
+                        )
                         .key_binding(
                             KeyBinding::for_action_in(&menu::Confirm, &focus_handle, cx)
                                 .map(|kb| kb.size(rems_from_px(12.))),
@@ -758,18 +783,38 @@ impl PickerDelegate for WorktreeListDelegate {
                         .on_click(|_, window, cx| {
                             window.dispatch_action(menu::Confirm.boxed_clone(), cx)
                         }),
-                )
-                .child(
-                    Button::new("open-in-window", "Open")
-                        .key_binding(
-                            KeyBinding::for_action_in(&menu::SecondaryConfirm, &focus_handle, cx)
+                    )
+                    .into_any(),
+            )
+        } else {
+            Some(
+                footer_container
+                    .child(
+                        Button::new("open-in-new-window", "Open in New Window")
+                            .key_binding(
+                                KeyBinding::for_action_in(&menu::Confirm, &focus_handle, cx)
+                                    .map(|kb| kb.size(rems_from_px(12.))),
+                            )
+                            .on_click(|_, window, cx| {
+                                window.dispatch_action(menu::Confirm.boxed_clone(), cx)
+                            }),
+                    )
+                    .child(
+                        Button::new("open-in-window", "Open")
+                            .key_binding(
+                                KeyBinding::for_action_in(
+                                    &menu::SecondaryConfirm,
+                                    &focus_handle,
+                                    cx,
+                                )
                                 .map(|kb| kb.size(rems_from_px(12.))),
-                        )
-                        .on_click(|_, window, cx| {
-                            window.dispatch_action(menu::SecondaryConfirm.boxed_clone(), cx)
-                        }),
-                )
-                .into_any(),
-        )
+                            )
+                            .on_click(|_, window, cx| {
+                                window.dispatch_action(menu::SecondaryConfirm.boxed_clone(), cx)
+                            }),
+                    )
+                    .into_any(),
+            )
+        }
     }
 }
