@@ -112,21 +112,6 @@ impl<T: Future> Future for WithTimeout<T> {
     }
 }
 
-#[cfg(any(test, feature = "test-support"))]
-/// Uses smol executor to run a given future no longer than the timeout specified.
-/// Note that this won't "rewind" on `cx.executor().advance_clock` call, truly waiting for the timeout to elapse.
-pub async fn smol_timeout<F, T>(timeout: Duration, f: F) -> Result<T, ()>
-where
-    F: Future<Output = T>,
-{
-    let timer = async {
-        smol::Timer::after(timeout).await;
-        Err(())
-    };
-    let future = async move { Ok(f.await) };
-    smol::future::FutureExt::race(timer, future).await
-}
-
 /// Increment the given atomic counter if it is not zero.
 /// Return the new value of the counter.
 pub(crate) fn atomic_incr_if_not_zero(counter: &AtomicUsize) -> usize {
