@@ -8,6 +8,7 @@ use strum::{EnumIter, EnumString, IntoStaticStr};
 )]
 #[strum(serialize_all = "snake_case")]
 pub enum IconName {
+    AcpRegistry,
     Ai,
     AiAnthropic,
     AiBedrock,
@@ -262,8 +263,6 @@ pub enum IconName {
     ZedAgent,
     ZedAgentTwo,
     ZedAssistant,
-    ZedBurnMode,
-    ZedBurnModeOn,
     ZedPredict,
     ZedPredictDisabled,
     ZedPredictDown,
