@@ -10,6 +10,5 @@ actions!(
 );
 
 pub async fn register_zed_scheme(cx: &AsyncApp) -> anyhow::Result<()> {
-    cx.update(|cx| cx.register_url_scheme(ZED_URL_SCHEME))?
-        .await
+    cx.update(|cx| cx.register_url_scheme(ZED_URL_SCHEME)).await
 }
