@@ -20,7 +20,7 @@ use ui::{
     ContextMenu, ContextMenuEntry, Indicator, PopoverMenu, PopoverMenuHandle, Tooltip, prelude::*,
 };
 
-use util::{ResultExt, rel_path::RelPath};
+use util::{ResultExt, paths::PathExt, rel_path::RelPath};
 use workspace::{StatusItemView, Workspace};
 
 use crate::lsp_log_view;
@@ -132,12 +132,20 @@ impl LanguageServerState {
             return menu;
         };
 
-        let server_versions = self
+        let server_metadata = self
             .lsp_store
             .update(cx, |lsp_store, _| {
                 lsp_store
                     .language_server_statuses()
-                    .map(|(server_id, status)| (server_id, status.server_version.clone()))
+                    .map(|(server_id, status)| {
+                        (
+                            server_id,
+                            (
+                                status.server_version.clone(),
+                                status.binary.as_ref().map(|b| b.path.clone()),
+                            ),
+                        )
+                    })
                     .collect::<HashMap<_, _>>()
             })
             .unwrap_or_default();
@@ -266,11 +274,26 @@ impl LanguageServerState {
                 .or_else(|| server_info.binary_status.as_ref()?.message.as_ref())
                 .cloned();
 
-            let server_version = server_versions
+            let (server_version, binary_path) = server_metadata
                 .get(&server_info.id)
-                .and_then(|version| version.clone());
+                .map(|(version, path)| {
+                    (
+                        version.clone(),
+                        path.as_ref()
+                            .map(|p| SharedString::from(p.compact().to_string_lossy().to_string())),
+                    )
+                })
+                .unwrap_or((None, None));
 
-            let metadata_label = match (&server_version, &message) {
+            let truncated_message = message.as_ref().and_then(|message| {
+                message
+                    .lines()
+                    .filter(|line| !line.trim().is_empty())
+                    .map(SharedString::new)
+                    .next()
+            });
+
+            let metadata_label = match (&server_version, &truncated_message) {
                 (None, None) => None,
                 (Some(version), None) => Some(SharedString::from(format!("v{}", version.as_ref()))),
                 (None, Some(message)) => Some(message.clone()),
@@ -308,7 +331,7 @@ impl LanguageServerState {
                                 let Some(create_buffer) = workspace_for_message
                                     .update(cx, |workspace, cx| {
                                         workspace.project().update(cx, |project, cx| {
-                                            project.create_buffer(false, cx)
+                                            project.create_buffer(None, false, cx)
                                         })
                                     })
                                     .ok()
@@ -334,7 +357,7 @@ impl LanguageServerState {
                                             cx,
                                         );
                                         buffer.set_capability(language::Capability::ReadOnly, cx);
-                                    })?;
+                                    });
 
                                     workspace.update(cx, |workspace, cx| {
                                         window_handle.update(cx, |_, window, cx| {
@@ -487,10 +510,13 @@ impl LanguageServerState {
 
                         submenu = submenu.separator().custom_row({
                             let metadata_label = metadata_label.clone();
+                            let binary_path = binary_path.clone();
                             move |_, _| {
                                 h_flex()
+                                    .id("metadata-container")
                                     .ml_neg_1()
                                     .gap_1()
+                                    .max_w(rems(164.))
                                     .child(
                                         Icon::new(IconName::Circle)
                                             .color(status_color)
@@ -511,9 +537,13 @@ impl LanguageServerState {
                                             .child(
                                                 Label::new(metadata)
                                                     .size(LabelSize::Small)
-                                                    .color(Color::Muted),
+                                                    .color(Color::Muted)
+                                                    .truncate(),
                                             )
                                     })
+                                    .when_some(binary_path.clone(), |el, path| {
+                                        el.tooltip(Tooltip::text(path))
+                                    })
                                     .into_any_element()
                             }
                         });
