@@ -1,3 +1,7 @@
+((comment) @injection.content
+ (#set! injection.language "comment")
+)
+
 (preproc_def
     value: (preproc_arg) @injection.content
     (#set! injection.language "c++"))
