@@ -79,16 +79,13 @@ impl NotificationStore {
                 let this = this.upgrade()?;
                 match status {
                     client::Status::Connected { .. } => {
-                        if let Some(task) = this
-                            .update(cx, |this, cx| this.handle_connect(cx))
-                            .log_err()?
-                        {
+                        if let Some(task) = this.update(cx, |this, cx| this.handle_connect(cx)) {
                             task.await.log_err()?;
                         }
                     }
-                    _ => this
-                        .update(cx, |this, cx| this.handle_disconnect(cx))
-                        .log_err()?,
+                    _ => {
+                        this.update(cx, |this, cx| this.handle_disconnect(cx));
+                    }
                 }
             }
             Some(())
@@ -161,7 +158,7 @@ impl NotificationStore {
                 .context("Notification store was dropped while loading notifications")?;
 
             let response = request.await?;
-            this.update(cx, |this, _| this.loaded_all_notifications = response.done)?;
+            this.update(cx, |this, _| this.loaded_all_notifications = response.done);
             Self::add_notifications(
                 this,
                 response.notifications,
@@ -212,8 +209,8 @@ impl NotificationStore {
     ) -> Result<()> {
         this.update(&mut cx, |this, cx| {
             this.splice_notifications([(envelope.payload.notification_id, None)], false, cx);
-            Ok(())
-        })?
+        });
+        Ok(())
     }
 
     async fn add_notifications(
@@ -259,10 +256,10 @@ impl NotificationStore {
             }
         }
 
-        let user_store = this.read_with(cx, |this, _| this.user_store.clone())?;
+        let user_store = this.read_with(cx, |this, _| this.user_store.clone());
 
         user_store
-            .update(cx, |store, cx| store.get_users(user_ids, cx))?
+            .update(cx, |store, cx| store.get_users(user_ids, cx))
             .await?;
         this.update(cx, |this, cx| {
             if options.clear_old {
@@ -285,8 +282,7 @@ impl NotificationStore {
                 options.is_new,
                 cx,
             );
-        })
-        .log_err();
+        });
 
         Ok(())
     }
