@@ -3,7 +3,7 @@ use std::borrow::Cow;
 use anyhow::{Result, bail};
 use async_trait::async_trait;
 use dap::{DapLocator, DebugRequest, adapters::DebugAdapterName};
-use gpui::SharedString;
+use gpui::{BackgroundExecutor, SharedString};
 
 use task::{DebugScenario, SpawnInTerminal, TaskTemplate, VariableName};
 
@@ -56,7 +56,7 @@ impl DapLocator for NodeLocator {
         })
     }
 
-    async fn run(&self, _: SpawnInTerminal) -> Result<DebugRequest> {
+    async fn run(&self, _: SpawnInTerminal, _executor: BackgroundExecutor) -> Result<DebugRequest> {
         bail!("JavaScript locator should not require DapLocator::run to be ran");
     }
 }
