@@ -40,13 +40,13 @@ impl EventEmitter<Event> for LogStore {}
 pub struct LogStore {
     on_headless_host: bool,
     projects: HashMap<WeakEntity<Project>, ProjectState>,
-    pub copilot_log_subscription: Option<lsp::Subscription>,
     pub language_servers: HashMap<LanguageServerId, LanguageServerState>,
     io_tx: mpsc::UnboundedSender<(LanguageServerId, IoKind, String)>,
 }
 
 struct ProjectState {
     _subscriptions: [Subscription; 2],
+    copilot_log_subscription: Option<lsp::Subscription>,
 }
 
 pub trait Message: AsRef<str> {
@@ -220,7 +220,7 @@ impl LogStore {
         let log_store = Self {
             projects: HashMap::default(),
             language_servers: HashMap::default(),
-            copilot_log_subscription: None,
+
             on_headless_host,
             io_tx,
         };
@@ -229,7 +229,7 @@ impl LogStore {
                 if let Some(log_store) = log_store.upgrade() {
                     log_store.update(cx, |log_store, cx| {
                         log_store.on_io(server_id, io_kind, &message, cx);
-                    })?;
+                    });
                 }
             }
             anyhow::Ok(())
@@ -350,6 +350,7 @@ impl LogStore {
                         }
                     }),
                 ],
+                copilot_log_subscription: None,
             },
         );
     }
@@ -713,4 +714,12 @@ impl LogStore {
             }
         }
     }
+    pub fn copilot_state_for_project(
+        &mut self,
+        project: &WeakEntity<Project>,
+    ) -> Option<&mut Option<lsp::Subscription>> {
+        self.projects
+            .get_mut(project)
+            .map(|project| &mut project.copilot_log_subscription)
+    }
 }
