@@ -270,7 +270,7 @@ impl ToolchainStore {
                 RelPath::empty().into()
             };
             Ok(this.activate_toolchain(ProjectPath { worktree_id, path }, toolchain, cx))
-        })??
+        })?
         .await;
         Ok(proto::Ack {})
     }
@@ -292,7 +292,7 @@ impl ToolchainStore {
                     language_name,
                     cx,
                 )
-            })?
+            })
             .await;
 
         Ok(proto::ActiveToolchainResponse {
@@ -322,7 +322,7 @@ impl ToolchainStore {
                     language_name,
                     cx,
                 ))
-            })??
+            })?
             .await;
         let has_values = toolchains.is_some();
         let groups = if let Some(Toolchains { toolchains, .. }) = &toolchains {
@@ -380,7 +380,7 @@ impl ToolchainStore {
                 let language_name = LanguageName::from_proto(envelope.payload.language_name);
                 let path = PathBuf::from(envelope.payload.abs_path);
                 this.resolve_toolchain(path, language_name, cx)
-            })?
+            })
             .await;
         let response = match toolchain {
             Ok(toolchain) => {
@@ -539,9 +539,7 @@ impl LocalToolchainStore {
                     path: Arc::from(RelPath::empty()),
                     worktree_id,
                 });
-            let abs_path = worktree
-                .update(cx, |this, _| this.absolutize(&relative_path.path))
-                .ok()?;
+            let abs_path = worktree.update(cx, |this, _| this.absolutize(&relative_path.path));
 
             let project_env = environment
                 .update(cx, |environment, cx| {
@@ -551,7 +549,6 @@ impl LocalToolchainStore {
                         cx,
                     )
                 })
-                .ok()?
                 .await;
 
             cx.background_spawn(async move {
@@ -613,7 +610,7 @@ impl LocalToolchainStore {
                         path.as_path().into(),
                         cx,
                     )
-                })?
+                })
                 .await;
             cx.background_spawn(async move {
                 toolchain_lister
