@@ -8,7 +8,7 @@ use askpass::EncryptedPassword;
 use auto_update::AutoUpdater;
 use editor::Editor;
 use extension_host::ExtensionStore;
-use futures::channel::oneshot;
+use futures::{FutureExt as _, channel::oneshot, select};
 use gpui::{
     AnyWindowHandle, App, AsyncApp, DismissEvent, Entity, EventEmitter, Focusable, FontFeatures,
     ParentElement as _, PromptLevel, Render, SharedString, Task, TextStyleRefinement, WeakEntity,
@@ -19,7 +19,7 @@ use markdown::{Markdown, MarkdownElement, MarkdownStyle};
 use project::trusted_worktrees;
 use release_channel::ReleaseChannel;
 use remote::{
-    ConnectionIdentifier, DockerConnectionOptions, RemoteClient, RemoteConnection,
+    ConnectionIdentifier, DockerConnectionOptions, Interactive, RemoteClient, RemoteConnection,
     RemoteConnectionOptions, RemotePlatform, SshConnectionOptions,
 };
 use semver::Version;
@@ -96,9 +96,10 @@ impl From<Connection> for RemoteConnectionOptions {
             Connection::Wsl(conn) => RemoteConnectionOptions::Wsl(conn.into()),
             Connection::DevContainer(conn) => {
                 RemoteConnectionOptions::Docker(DockerConnectionOptions {
-                    name: conn.name.to_string(),
-                    container_id: conn.container_id.to_string(),
+                    name: conn.name,
+                    container_id: conn.container_id,
                     upload_binary_over_docker_exec: false,
+                    use_podman: conn.use_podman,
                 })
             }
         }
@@ -142,6 +143,7 @@ pub struct RemoteConnectionPrompt {
 impl Drop for RemoteConnectionPrompt {
     fn drop(&mut self) {
         if let Some(cancel) = self.cancellation.take() {
+            log::debug!("cancelling remote connection");
             cancel.send(()).ok();
         }
     }
@@ -309,6 +311,10 @@ impl RemoteConnectionModal {
                 (options.distro_name.clone(), None, true, false)
             }
             RemoteConnectionOptions::Docker(options) => (options.name.clone(), None, false, true),
+            #[cfg(any(test, feature = "test-support"))]
+            RemoteConnectionOptions::Mock(options) => {
+                (format!("mock-{}", options.id), None, false, false)
+            }
         };
         Self {
             prompt: cx.new(|cx| {
@@ -341,6 +347,7 @@ impl RemoteConnectionModal {
             .prompt
             .update(cx, |prompt, _cx| prompt.cancellation.take())
         {
+            log::debug!("cancelling remote connection");
             tx.send(()).ok();
         }
         self.finished(cx);
@@ -575,13 +582,13 @@ impl remote::RemoteClientDelegate for RemoteClientDelegate {
 
 impl RemoteClientDelegate {
     fn update_status(&self, status: Option<&str>, cx: &mut AsyncApp) {
-        self.window
-            .update(cx, |_, _, cx| {
-                self.ui.update(cx, |modal, cx| {
+        cx.update(|cx| {
+            self.ui
+                .update(cx, |modal, cx| {
                     modal.set_status(status.map(|s| s.to_string()), cx);
                 })
-            })
-            .ok();
+                .ok()
+        });
     }
 }
 
@@ -600,7 +607,7 @@ pub fn connect(
             .and_then(|pw| pw.try_into().ok()),
         _ => None,
     };
-    let (tx, rx) = oneshot::channel();
+    let (tx, mut rx) = oneshot::channel();
     ui.update(cx, |ui, _cx| ui.set_cancellation_tx(tx));
 
     let delegate = Arc::new(RemoteClientDelegate {
@@ -610,8 +617,13 @@ pub fn connect(
     });
 
     cx.spawn(async move |cx| {
-        let connection = remote::connect(connection_options, delegate.clone(), cx).await?;
-        cx.update(|cx| remote::RemoteClient::new(unique_identifier, connection, rx, delegate, cx))?
+        let connection = remote::connect(connection_options, delegate.clone(), cx);
+        let connection = select! {
+            _ = rx => return Ok(None),
+            result = connection.fuse() => result,
+        }?;
+
+        cx.update(|cx| remote::RemoteClient::new(unique_identifier, connection, rx, delegate, cx))
             .await
     })
 }
@@ -631,12 +643,12 @@ pub async fn open_remote_project(
             .update(|cx| {
                 // todo: These paths are wrong they may have column and line information
                 workspace::remote_workspace_position_from_db(connection_options.clone(), &paths, cx)
-            })?
+            })
             .await
             .context("fetching remote workspace position from db")?;
 
         let mut options =
-            cx.update(|cx| (app_state.build_window_options)(workspace_position.display, cx))?;
+            cx.update(|cx| (app_state.build_window_options)(workspace_position.display, cx));
         options.window_bounds = workspace_position.window_bounds;
 
         cx.open_window(options, |window, cx| {
@@ -647,7 +659,10 @@ pub async fn open_remote_project(
                 app_state.languages.clone(),
                 app_state.fs.clone(),
                 None,
-                false,
+                project::LocalProjectFlags {
+                    init_worktree_trust: false,
+                    ..Default::default()
+                },
                 cx,
             );
             cx.new(|cx| {
@@ -659,12 +674,13 @@ pub async fn open_remote_project(
     };
 
     loop {
-        let (cancel_tx, cancel_rx) = oneshot::channel();
+        let (cancel_tx, mut cancel_rx) = oneshot::channel();
         let delegate = window.update(cx, {
             let paths = paths.clone();
             let connection_options = connection_options.clone();
             move |workspace, window, cx| {
                 window.activate_window();
+                workspace.hide_modal(window, cx);
                 workspace.toggle_modal(window, cx, |window, cx| {
                     RemoteConnectionModal::new(&connection_options, paths, window, cx)
                 });
@@ -698,48 +714,66 @@ pub async fn open_remote_project(
 
         let Some(delegate) = delegate else { break };
 
-        let remote_connection =
-            match remote::connect(connection_options.clone(), delegate.clone(), cx).await {
-                Ok(connection) => connection,
-                Err(e) => {
-                    window
-                        .update(cx, |workspace, _, cx| {
-                            if let Some(ui) = workspace.active_modal::<RemoteConnectionModal>(cx) {
-                                ui.update(cx, |modal, cx| modal.finished(cx))
-                            }
-                        })
-                        .ok();
-                    log::error!("Failed to open project: {e:#}");
-                    let response = window
-                        .update(cx, |_, window, cx| {
-                            window.prompt(
-                                PromptLevel::Critical,
-                                match connection_options {
-                                    RemoteConnectionOptions::Ssh(_) => "Failed to connect over SSH",
-                                    RemoteConnectionOptions::Wsl(_) => "Failed to connect to WSL",
-                                    RemoteConnectionOptions::Docker(_) => {
-                                        "Failed to connect to Dev Container"
-                                    }
-                                },
-                                Some(&format!("{e:#}")),
-                                &["Retry", "Cancel"],
-                                cx,
-                            )
-                        })?
-                        .await;
+        let connection = remote::connect(connection_options.clone(), delegate.clone(), cx);
+        let connection = select! {
+            _ = cancel_rx => {
+                window
+                    .update(cx, |workspace, _, cx| {
+                        if let Some(ui) = workspace.active_modal::<RemoteConnectionModal>(cx) {
+                            ui.update(cx, |modal, cx| modal.finished(cx))
+                        }
+                    })
+                    .ok();
 
-                    if response == Ok(0) {
-                        continue;
-                    }
+                break;
+            },
+            result = connection.fuse() => result,
+        };
+        let remote_connection = match connection {
+            Ok(connection) => connection,
+            Err(e) => {
+                window
+                    .update(cx, |workspace, _, cx| {
+                        if let Some(ui) = workspace.active_modal::<RemoteConnectionModal>(cx) {
+                            ui.update(cx, |modal, cx| modal.finished(cx))
+                        }
+                    })
+                    .ok();
+                log::error!("Failed to open project: {e:#}");
+                let response = window
+                    .update(cx, |_, window, cx| {
+                        window.prompt(
+                            PromptLevel::Critical,
+                            match connection_options {
+                                RemoteConnectionOptions::Ssh(_) => "Failed to connect over SSH",
+                                RemoteConnectionOptions::Wsl(_) => "Failed to connect to WSL",
+                                RemoteConnectionOptions::Docker(_) => {
+                                    "Failed to connect to Dev Container"
+                                }
+                                #[cfg(any(test, feature = "test-support"))]
+                                RemoteConnectionOptions::Mock(_) => {
+                                    "Failed to connect to mock server"
+                                }
+                            },
+                            Some(&format!("{e:#}")),
+                            &["Retry", "Cancel"],
+                            cx,
+                        )
+                    })?
+                    .await;
 
-                    if created_new_window {
-                        window
-                            .update(cx, |_, window, _| window.remove_window())
-                            .ok();
-                    }
-                    break;
+                if response == Ok(0) {
+                    continue;
                 }
-            };
+
+                if created_new_window {
+                    window
+                        .update(cx, |_, window, _| window.remove_window())
+                        .ok();
+                }
+                return Ok(());
+            }
+        };
 
         let (paths, paths_with_positions) =
             determine_paths_with_positions(&remote_connection, paths.clone()).await;
@@ -755,7 +789,7 @@ pub async fn open_remote_project(
                     paths.clone(),
                     cx,
                 )
-            })?
+            })
             .await;
 
         window
@@ -779,6 +813,10 @@ pub async fn open_remote_project(
                                 RemoteConnectionOptions::Docker(_) => {
                                     "Failed to connect to Dev Container"
                                 }
+                                #[cfg(any(test, feature = "test-support"))]
+                                RemoteConnectionOptions::Mock(_) => {
+                                    "Failed to connect to mock server"
+                                }
                             },
                             Some(&format!("{e:#}")),
                             &["Retry", "Cancel"],
@@ -833,18 +871,19 @@ pub async fn open_remote_project(
             }
         }
 
-        window
-            .update(cx, |workspace, _, cx| {
-                if let Some(client) = workspace.project().read(cx).remote_client() {
-                    ExtensionStore::global(cx)
-                        .update(cx, |store, cx| store.register_remote_client(client, cx));
-                }
-            })
-            .ok();
-
         break;
     }
 
+    window
+        .update(cx, |workspace, _, cx| {
+            if let Some(client) = workspace.project().read(cx).remote_client() {
+                if let Some(extension_store) = ExtensionStore::try_global(cx) {
+                    extension_store
+                        .update(cx, |store, cx| store.register_remote_client(client, cx));
+                }
+            }
+        })
+        .ok();
     // Already showed the error to the user
     Ok(())
 }
@@ -877,6 +916,7 @@ async fn path_exists(connection: &Arc<dyn RemoteConnection>, path: &Path) -> boo
         &Default::default(),
         None,
         None,
+        Interactive::No,
     ) else {
         return false;
     };
@@ -889,3 +929,102 @@ async fn path_exists(connection: &Arc<dyn RemoteConnection>, path: &Path) -> boo
     };
     child.status().await.is_ok_and(|status| status.success())
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use extension::ExtensionHostProxy;
+    use fs::FakeFs;
+    use gpui::TestAppContext;
+    use http_client::BlockedHttpClient;
+    use node_runtime::NodeRuntime;
+    use remote::RemoteClient;
+    use remote_server::{HeadlessAppState, HeadlessProject};
+    use serde_json::json;
+    use util::path;
+
+    #[gpui::test]
+    async fn test_open_remote_project_with_mock_connection(
+        cx: &mut TestAppContext,
+        server_cx: &mut TestAppContext,
+    ) {
+        let app_state = init_test(cx);
+        let executor = cx.executor();
+
+        cx.update(|cx| {
+            release_channel::init(semver::Version::new(0, 0, 0), cx);
+        });
+        server_cx.update(|cx| {
+            release_channel::init(semver::Version::new(0, 0, 0), cx);
+        });
+
+        let (opts, server_session, connect_guard) = RemoteClient::fake_server(cx, server_cx);
+
+        let remote_fs = FakeFs::new(server_cx.executor());
+        remote_fs
+            .insert_tree(
+                path!("/project"),
+                json!({
+                    "src": {
+                        "main.rs": "fn main() {}",
+                    },
+                    "README.md": "# Test Project",
+                }),
+            )
+            .await;
+
+        server_cx.update(HeadlessProject::init);
+        let http_client = Arc::new(BlockedHttpClient);
+        let node_runtime = NodeRuntime::unavailable();
+        let languages = Arc::new(language::LanguageRegistry::new(server_cx.executor()));
+        let proxy = Arc::new(ExtensionHostProxy::new());
+
+        let _headless = server_cx.new(|cx| {
+            HeadlessProject::new(
+                HeadlessAppState {
+                    session: server_session,
+                    fs: remote_fs.clone(),
+                    http_client,
+                    node_runtime,
+                    languages,
+                    extension_host_proxy: proxy,
+                },
+                false,
+                cx,
+            )
+        });
+
+        drop(connect_guard);
+
+        let paths = vec![PathBuf::from(path!("/project"))];
+        let open_options = workspace::OpenOptions::default();
+
+        let mut async_cx = cx.to_async();
+        let result = open_remote_project(opts, paths, app_state, open_options, &mut async_cx).await;
+
+        executor.run_until_parked();
+
+        assert!(result.is_ok(), "open_remote_project should succeed");
+
+        let windows = cx.update(|cx| cx.windows().len());
+        assert_eq!(windows, 1, "Should have opened a window");
+
+        let workspace_handle = cx.update(|cx| cx.windows()[0].downcast::<Workspace>().unwrap());
+
+        workspace_handle
+            .update(cx, |workspace, _, cx| {
+                let project = workspace.project().read(cx);
+                assert!(project.is_remote(), "Project should be a remote project");
+            })
+            .unwrap();
+    }
+
+    fn init_test(cx: &mut TestAppContext) -> Arc<AppState> {
+        cx.update(|cx| {
+            let state = AppState::test(cx);
+            crate::init(cx);
+            editor::init(cx);
+            state
+        })
+    }
+}
