@@ -34,13 +34,10 @@
 //! interpreting and displaying various types of Jupyter output.
 
 use editor::{Editor, MultiBuffer};
-use gpui::{AnyElement, ClipboardItem, Entity, Render, WeakEntity};
+use gpui::{AnyElement, ClipboardItem, Entity, EventEmitter, Render, WeakEntity};
 use language::Buffer;
 use runtimelib::{ExecutionState, JupyterMessageContent, MimeBundle, MimeType};
-use ui::{
-    ButtonStyle, CommonAnimationExt, Context, IconButton, IconName, IntoElement, Styled, Tooltip,
-    Window, div, h_flex, prelude::*, v_flex,
-};
+use ui::{CommonAnimationExt, CopyButton, IconButton, Tooltip, prelude::*};
 
 mod image;
 use image::ImageView;
@@ -58,6 +55,9 @@ pub(crate) mod user_error;
 use user_error::ErrorView;
 use workspace::Workspace;
 
+use crate::repl_settings::ReplSettings;
+use settings::Settings;
+
 /// When deciding what to render from a collection of mediatypes, we need to rank them in order of importance
 fn rank_mime_type(mimetype: &MimeType) -> usize {
     match mimetype {
@@ -236,89 +236,62 @@ impl Output {
                 Self::Image { content, .. } => {
                     Self::render_output_controls(content.clone(), workspace, window, cx)
                 }
-                Self::ErrorOutput(err) => {
-                    // Add buttons for the traceback section
-                    Some(
-                        h_flex()
-                            .pl_1()
-                            .child(
-                                IconButton::new(
-                                    ElementId::Name("copy-full-error-traceback".into()),
-                                    IconName::Copy,
-                                )
-                                .style(ButtonStyle::Transparent)
-                                .tooltip(Tooltip::text("Copy Full Error"))
-                                .on_click({
-                                    let ename = err.ename.clone();
-                                    let evalue = err.evalue.clone();
-                                    let traceback = err.traceback.clone();
-                                    move |_, _window, cx| {
+                Self::ErrorOutput(err) => Some(
+                    h_flex()
+                        .pl_1()
+                        .child({
+                            let ename = err.ename.clone();
+                            let evalue = err.evalue.clone();
+                            let traceback = err.traceback.clone();
+                            let traceback_text = traceback.read(cx).full_text();
+                            let full_error = format!("{}: {}\n{}", ename, evalue, traceback_text);
+
+                            CopyButton::new(full_error).tooltip_label("Copy Full Error")
+                        })
+                        .child(
+                            IconButton::new(
+                                ElementId::Name("open-full-error-in-buffer-traceback".into()),
+                                IconName::FileTextOutlined,
+                            )
+                            .style(ButtonStyle::Transparent)
+                            .tooltip(Tooltip::text("Open Full Error in Buffer"))
+                            .on_click({
+                                let ename = err.ename.clone();
+                                let evalue = err.evalue.clone();
+                                let traceback = err.traceback.clone();
+                                move |_, window, cx| {
+                                    if let Some(workspace) = workspace.upgrade() {
                                         let traceback_text = traceback.read(cx).full_text();
                                         let full_error =
                                             format!("{}: {}\n{}", ename, evalue, traceback_text);
-                                        let clipboard_content =
-                                            ClipboardItem::new_string(full_error);
-                                        cx.write_to_clipboard(clipboard_content);
-                                    }
-                                }),
-                            )
-                            .child(
-                                IconButton::new(
-                                    ElementId::Name("open-full-error-in-buffer-traceback".into()),
-                                    IconName::FileTextOutlined,
-                                )
-                                .style(ButtonStyle::Transparent)
-                                .tooltip(Tooltip::text("Open Full Error in Buffer"))
-                                .on_click({
-                                    let ename = err.ename.clone();
-                                    let evalue = err.evalue.clone();
-                                    let traceback = err.traceback.clone();
-                                    move |_, window, cx| {
-                                        if let Some(workspace) = workspace.upgrade() {
-                                            let traceback_text = traceback.read(cx).full_text();
-                                            let full_error = format!(
-                                                "{}: {}\n{}",
-                                                ename, evalue, traceback_text
+                                        let buffer = cx.new(|cx| {
+                                            let mut buffer = Buffer::local(full_error, cx)
+                                                .with_language(language::PLAIN_TEXT.clone(), cx);
+                                            buffer
+                                                .set_capability(language::Capability::ReadOnly, cx);
+                                            buffer
+                                        });
+                                        let editor = Box::new(cx.new(|cx| {
+                                            let multibuffer = cx.new(|cx| {
+                                                let mut multi_buffer =
+                                                    MultiBuffer::singleton(buffer.clone(), cx);
+                                                multi_buffer
+                                                    .set_title("Full Error".to_string(), cx);
+                                                multi_buffer
+                                            });
+                                            Editor::for_multibuffer(multibuffer, None, window, cx)
+                                        }));
+                                        workspace.update(cx, |workspace, cx| {
+                                            workspace.add_item_to_active_pane(
+                                                editor, None, true, window, cx,
                                             );
-                                            let buffer = cx.new(|cx| {
-                                                let mut buffer = Buffer::local(full_error, cx)
-                                                    .with_language(
-                                                        language::PLAIN_TEXT.clone(),
-                                                        cx,
-                                                    );
-                                                buffer.set_capability(
-                                                    language::Capability::ReadOnly,
-                                                    cx,
-                                                );
-                                                buffer
-                                            });
-                                            let editor = Box::new(cx.new(|cx| {
-                                                let multibuffer = cx.new(|cx| {
-                                                    let mut multi_buffer =
-                                                        MultiBuffer::singleton(buffer.clone(), cx);
-                                                    multi_buffer
-                                                        .set_title("Full Error".to_string(), cx);
-                                                    multi_buffer
-                                                });
-                                                Editor::for_multibuffer(
-                                                    multibuffer,
-                                                    None,
-                                                    window,
-                                                    cx,
-                                                )
-                                            }));
-                                            workspace.update(cx, |workspace, cx| {
-                                                workspace.add_item_to_active_pane(
-                                                    editor, None, true, window, cx,
-                                                );
-                                            });
-                                        }
+                                        });
                                     }
-                                }),
-                            )
-                            .into_any_element(),
-                    )
-                }
+                                }
+                            }),
+                        )
+                        .into_any_element(),
+                ),
                 Self::Message(_) => None,
                 Self::Table { content, .. } => {
                     Self::render_output_controls(content.clone(), workspace, window, cx)
@@ -389,6 +362,9 @@ pub enum ExecutionStatus {
     Restarting,
 }
 
+pub struct ExecutionViewFinishedEmpty;
+pub struct ExecutionViewFinishedSmall(pub String);
+
 /// An ExecutionView shows the outputs of an execution.
 /// It can hold zero or more outputs, which the user
 /// sees as "the output" for a single execution.
@@ -399,6 +375,9 @@ pub struct ExecutionView {
     pub status: ExecutionStatus,
 }
 
+impl EventEmitter<ExecutionViewFinishedEmpty> for ExecutionView {}
+impl EventEmitter<ExecutionViewFinishedSmall> for ExecutionView {}
+
 impl ExecutionView {
     pub fn new(
         status: ExecutionStatus,
@@ -475,7 +454,16 @@ impl ExecutionView {
                     ExecutionState::Busy => {
                         self.status = ExecutionStatus::Executing;
                     }
-                    ExecutionState::Idle => self.status = ExecutionStatus::Finished,
+                    ExecutionState::Idle => {
+                        self.status = ExecutionStatus::Finished;
+                        if self.outputs.is_empty() {
+                            cx.emit(ExecutionViewFinishedEmpty);
+                        } else if ReplSettings::get_global(cx).inline_output {
+                            if let Some(small_text) = self.get_small_inline_output(cx) {
+                                cx.emit(ExecutionViewFinishedSmall(small_text));
+                            }
+                        }
+                    }
                     ExecutionState::Unknown => self.status = ExecutionStatus::Unknown,
                     ExecutionState::Starting => self.status = ExecutionStatus::ConnectingToKernel,
                     ExecutionState::Restarting => self.status = ExecutionStatus::Restarting,
@@ -527,6 +515,35 @@ impl ExecutionView {
         }
     }
 
+    /// Check if the output is a single small plain text that can be shown inline.
+    /// Returns the text if it's suitable for inline display (single line, short enough).
+    fn get_small_inline_output(&self, cx: &App) -> Option<String> {
+        // Only consider single outputs
+        if self.outputs.len() != 1 {
+            return None;
+        }
+
+        let output = self.outputs.first()?;
+
+        // Only Plain outputs can be inlined
+        let content = match output {
+            Output::Plain { content, .. } => content,
+            _ => return None,
+        };
+
+        let text = content.read(cx).full_text();
+        let trimmed = text.trim();
+
+        let max_length = ReplSettings::get_global(cx).inline_output_max_length;
+
+        // Must be a single line and within the configured max length
+        if trimmed.contains('\n') || trimmed.len() > max_length {
+            return None;
+        }
+
+        Some(trimmed.to_string())
+    }
+
     fn apply_terminal_text(
         &mut self,
         text: &str,
