@@ -1,5 +1,3 @@
-use std::time::Duration;
-
 use db::kvp::KEY_VALUE_STORE;
 use gpui::{App, AppContext as _, Context, Subscription, Task, WindowId};
 use util::ResultExt;
@@ -64,22 +62,30 @@ impl AppSession {
     pub fn new(session: Session, cx: &Context<Self>) -> Self {
         let _subscriptions = vec![cx.on_app_quit(Self::app_will_quit)];
 
+        #[cfg(not(any(test, feature = "test-support")))]
         let _serialization_task = cx.spawn(async move |_, cx| {
-            let mut current_window_stack = Vec::new();
-            loop {
-                if let Some(windows) = cx.update(|cx| window_stack(cx)).ok().flatten()
-                    && windows != current_window_stack
-                {
-                    store_window_stack(&windows).await;
-                    current_window_stack = windows;
-                }
+            // Disabled in tests: the infinite loop bypasses "parking forbidden" checks,
+            // causing tests to hang instead of panicking.
+            {
+                let mut current_window_stack = Vec::new();
+                loop {
+                    if let Some(windows) = cx.update(|cx| window_stack(cx))
+                        && windows != current_window_stack
+                    {
+                        store_window_stack(&windows).await;
+                        current_window_stack = windows;
+                    }
 
-                cx.background_executor()
-                    .timer(Duration::from_millis(500))
-                    .await;
+                    cx.background_executor()
+                        .timer(std::time::Duration::from_millis(500))
+                        .await;
+                }
             }
         });
 
+        #[cfg(any(test, feature = "test-support"))]
+        let _serialization_task = Task::ready(());
+
         Self {
             session,
             _subscriptions,
