@@ -5,7 +5,14 @@ pub mod svg_preview_view;
 
 actions!(
     svg,
-    [OpenPreview, OpenPreviewToTheSide, OpenFollowingPreview]
+    [
+        /// Opens an SVG preview for the current file.
+        OpenPreview,
+        /// Opens an SVG preview in a split pane.
+        OpenPreviewToTheSide,
+        /// Opens a following SVG preview that syncs with the editor.
+        OpenFollowingPreview
+    ]
 );
 
 pub fn init(cx: &mut App) {
