@@ -17,7 +17,6 @@ mod icon;
 mod image;
 mod indent_guides;
 mod indicator;
-mod inline_code;
 mod keybinding;
 mod keybinding_hint;
 mod label;
@@ -64,7 +63,6 @@ pub use icon::*;
 pub use image::*;
 pub use indent_guides::*;
 pub use indicator::*;
-pub use inline_code::*;
 pub use keybinding::*;
 pub use keybinding_hint::*;
 pub use label::*;
