@@ -2,8 +2,10 @@ mod highlighted_label;
 mod label;
 mod label_like;
 mod loading_label;
+mod spinner_label;
 
 pub use highlighted_label::*;
 pub use label::*;
 pub use label_like::*;
 pub use loading_label::*;
+pub use spinner_label::*;
