@@ -10,6 +10,64 @@ use language::BracketPair;
 
 use std::sync::Arc;
 
+/// A char-based surround pair definition.
+/// Single source of truth for all supported surround pairs.
+#[derive(Clone, Copy, Debug, PartialEq)]
+pub struct SurroundPair {
+    pub open: char,
+    pub close: char,
+}
+
+impl SurroundPair {
+    pub const fn new(open: char, close: char) -> Self {
+        Self { open, close }
+    }
+
+    pub fn to_bracket_pair(self) -> BracketPair {
+        BracketPair {
+            start: self.open.to_string(),
+            end: self.close.to_string(),
+            close: true,
+            surround: true,
+            newline: false,
+        }
+    }
+
+    pub fn to_object(self) -> Option<Object> {
+        match self.open {
+            '\'' => Some(Object::Quotes),
+            '`' => Some(Object::BackQuotes),
+            '"' => Some(Object::DoubleQuotes),
+            '|' => Some(Object::VerticalBars),
+            '(' => Some(Object::Parentheses),
+            '[' => Some(Object::SquareBrackets),
+            '{' => Some(Object::CurlyBrackets),
+            '<' => Some(Object::AngleBrackets),
+            _ => None,
+        }
+    }
+}
+
+/// All supported surround pairs - single source of truth.
+pub const SURROUND_PAIRS: &[SurroundPair] = &[
+    SurroundPair::new('(', ')'),
+    SurroundPair::new('[', ']'),
+    SurroundPair::new('{', '}'),
+    SurroundPair::new('<', '>'),
+    SurroundPair::new('"', '"'),
+    SurroundPair::new('\'', '\''),
+    SurroundPair::new('`', '`'),
+    SurroundPair::new('|', '|'),
+];
+
+/// Bracket-only pairs for AnyBrackets matching.
+const BRACKET_PAIRS: &[SurroundPair] = &[
+    SurroundPair::new('(', ')'),
+    SurroundPair::new('[', ']'),
+    SurroundPair::new('{', '}'),
+    SurroundPair::new('<', '>'),
+];
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum SurroundsType {
     Motion(Motion),
@@ -34,16 +92,7 @@ impl Vim {
             editor.transact(window, cx, |editor, window, cx| {
                 editor.set_clip_at_line_ends(false, cx);
 
-                let pair = match find_surround_pair(&all_support_surround_pair(), &text) {
-                    Some(pair) => pair.clone(),
-                    None => BracketPair {
-                        start: text.to_string(),
-                        end: text.to_string(),
-                        close: true,
-                        surround: true,
-                        newline: false,
-                    },
-                };
+                let pair = bracket_pair_for_str_vim(&text);
                 let surround = pair.end != surround_alias((*text).as_ref());
                 let display_map = editor.display_snapshot(cx);
                 let display_selections = editor.selections.all_adjusted_display(&display_map);
@@ -131,14 +180,16 @@ impl Vim {
         self.stop_recording(cx);
 
         // only legitimate surrounds can be removed
-        let pair = match find_surround_pair(&all_support_surround_pair(), &text) {
-            Some(pair) => pair.clone(),
-            None => return,
+        let Some(first_char) = text.chars().next() else {
+            return;
         };
-        let pair_object = match pair_to_object(&pair) {
-            Some(pair_object) => pair_object,
-            None => return,
+        let Some(surround_pair) = surround_pair_for_char_vim(first_char) else {
+            return;
         };
+        let Some(pair_object) = surround_pair.to_object() else {
+            return;
+        };
+        let pair = surround_pair.to_bracket_pair();
         let surround = pair.end != *text;
 
         self.update_editor(cx, |_, editor, cx| {
@@ -233,16 +284,7 @@ impl Vim {
                 editor.transact(window, cx, |editor, window, cx| {
                     editor.set_clip_at_line_ends(false, cx);
 
-                    let pair = match find_surround_pair(&all_support_surround_pair(), &text) {
-                        Some(pair) => pair.clone(),
-                        None => BracketPair {
-                            start: text.to_string(),
-                            end: text.to_string(),
-                            close: true,
-                            surround: true,
-                            newline: false,
-                        },
-                    };
+                    let pair = bracket_pair_for_str_vim(&text);
 
                     // A single space should be added if the new surround is a
                     // bracket and not a quote (pair.start != pair.end) and if
@@ -437,144 +479,91 @@ impl Vim {
         object: Object,
         cx: &mut Context<Self>,
     ) -> Option<BracketPair> {
-        match object {
-            Object::Quotes => Some(BracketPair {
-                start: "'".to_string(),
-                end: "'".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::BackQuotes => Some(BracketPair {
-                start: "`".to_string(),
-                end: "`".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::DoubleQuotes => Some(BracketPair {
-                start: "\"".to_string(),
-                end: "\"".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::VerticalBars => Some(BracketPair {
-                start: "|".to_string(),
-                end: "|".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::Parentheses => Some(BracketPair {
-                start: "(".to_string(),
-                end: ")".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::SquareBrackets => Some(BracketPair {
-                start: "[".to_string(),
-                end: "]".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::CurlyBrackets { .. } => Some(BracketPair {
-                start: "{".to_string(),
-                end: "}".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::AngleBrackets => Some(BracketPair {
-                start: "<".to_string(),
-                end: ">".to_string(),
-                close: true,
-                surround: true,
-                newline: false,
-            }),
-            Object::AnyBrackets => {
-                // If we're dealing with `AnyBrackets`, which can map to multiple
-                // bracket pairs, we'll need to first determine which `BracketPair` to
-                // target.
-                // As such, we keep track of the smallest range size, so
-                // that in cases like `({ name: "John" })` if the cursor is
-                // inside the curly brackets, we target the curly brackets
-                // instead of the parentheses.
-                let mut bracket_pair = None;
-                let mut min_range_size = usize::MAX;
+        if let Some(pair) = object_to_surround_pair(object) {
+            return Some(pair.to_bracket_pair());
+        }
 
-                let _ = self.editor.update(cx, |editor, cx| {
-                    let display_map = editor.display_snapshot(cx);
-                    let selections = editor.selections.all_adjusted_display(&display_map);
-                    // Even if there's multiple cursors, we'll simply rely on
-                    // the first one to understand what bracket pair to map to.
-                    // I believe we could, if worth it, go one step above and
-                    // have a `BracketPair` per selection, so that `AnyBracket`
-                    // could work in situations where the transformation below
-                    // could be done.
-                    //
-                    // ```
-                    // (< name:ˇ'Zed' >)
-                    // <[ name:ˇ'DeltaDB' ]>
-                    // ```
-                    //
-                    // After using `csb{`:
-                    //
-                    // ```
-                    // (ˇ{ name:'Zed' })
-                    // <ˇ{ name:'DeltaDB' }>
-                    // ```
-                    if let Some(selection) = selections.first() {
-                        let relative_to = selection.head();
-                        let bracket_pairs = [('(', ')'), ('[', ']'), ('{', '}'), ('<', '>')];
-                        let cursor_offset = relative_to.to_offset(&display_map, Bias::Left);
+        if object != Object::AnyBrackets {
+            return None;
+        }
 
-                        for &(open, close) in bracket_pairs.iter() {
-                            if let Some(range) = surrounding_markers(
-                                &display_map,
-                                relative_to,
-                                true,
-                                false,
-                                open,
-                                close,
-                            ) {
-                                let start_offset = range.start.to_offset(&display_map, Bias::Left);
-                                let end_offset = range.end.to_offset(&display_map, Bias::Right);
+        // If we're dealing with `AnyBrackets`, which can map to multiple bracket
+        // pairs, we'll need to first determine which `BracketPair` to target.
+        // As such, we keep track of the smallest range size, so that in cases
+        // like `({ name: "John" })` if the cursor is inside the curly brackets,
+        // we target the curly brackets instead of the parentheses.
+        let mut best_pair = None;
+        let mut min_range_size = usize::MAX;
 
-                                if cursor_offset >= start_offset && cursor_offset <= end_offset {
-                                    let size = end_offset - start_offset;
-                                    if size < min_range_size {
-                                        min_range_size = size;
-                                        bracket_pair = Some(BracketPair {
-                                            start: open.to_string(),
-                                            end: close.to_string(),
-                                            close: true,
-                                            surround: true,
-                                            newline: false,
-                                        })
-                                    }
-                                }
+        let _ = self.editor.update(cx, |editor, cx| {
+            let display_map = editor.display_snapshot(cx);
+            let selections = editor.selections.all_adjusted_display(&display_map);
+            // Even if there's multiple cursors, we'll simply rely on the first one
+            // to understand what bracket pair to map to. I believe we could, if
+            // worth it, go one step above and have a `BracketPair` per selection, so
+            // that `AnyBracket` could work in situations where the transformation
+            // below could be done.
+            //
+            // ```
+            // (< name:ˇ'Zed' >)
+            // <[ name:ˇ'DeltaDB' ]>
+            // ```
+            //
+            // After using `csb{`:
+            //
+            // ```
+            // (ˇ{ name:'Zed' })
+            // <ˇ{ name:'DeltaDB' }>
+            // ```
+            if let Some(selection) = selections.first() {
+                let relative_to = selection.head();
+                let cursor_offset = relative_to.to_offset(&display_map, Bias::Left);
+
+                for pair in BRACKET_PAIRS {
+                    if let Some(range) = surrounding_markers(
+                        &display_map,
+                        relative_to,
+                        true,
+                        false,
+                        pair.open,
+                        pair.close,
+                    ) {
+                        let start_offset = range.start.to_offset(&display_map, Bias::Left);
+                        let end_offset = range.end.to_offset(&display_map, Bias::Right);
+
+                        if cursor_offset >= start_offset && cursor_offset <= end_offset {
+                            let size = end_offset - start_offset;
+                            if size < min_range_size {
+                                min_range_size = size;
+                                best_pair = Some(*pair);
                             }
                         }
                     }
-                });
-
-                bracket_pair
+                }
             }
-            _ => None,
-        }
+        });
+
+        best_pair.map(|p| p.to_bracket_pair())
     }
 }
 
-fn find_surround_pair<'a>(pairs: &'a [BracketPair], ch: &str) -> Option<&'a BracketPair> {
-    pairs
-        .iter()
-        .find(|pair| pair.start == surround_alias(ch) || pair.end == surround_alias(ch))
+/// Convert an Object to its corresponding SurroundPair.
+fn object_to_surround_pair(object: Object) -> Option<SurroundPair> {
+    let open = match object {
+        Object::Quotes => '\'',
+        Object::BackQuotes => '`',
+        Object::DoubleQuotes => '"',
+        Object::VerticalBars => '|',
+        Object::Parentheses => '(',
+        Object::SquareBrackets => '[',
+        Object::CurlyBrackets { .. } => '{',
+        Object::AngleBrackets => '<',
+        _ => return None,
+    };
+    surround_pair_for_char_vim(open)
 }
 
-fn surround_alias(ch: &str) -> &str {
+pub fn surround_alias(ch: &str) -> &str {
     match ch {
         "b" => ")",
         "B" => "}",
@@ -584,79 +573,65 @@ fn surround_alias(ch: &str) -> &str {
     }
 }
 
-fn all_support_surround_pair() -> Vec<BracketPair> {
-    vec![
-        BracketPair {
-            start: "{".into(),
-            end: "}".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-        BracketPair {
-            start: "'".into(),
-            end: "'".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-        BracketPair {
-            start: "`".into(),
-            end: "`".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-        BracketPair {
-            start: "\"".into(),
-            end: "\"".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-        BracketPair {
-            start: "(".into(),
-            end: ")".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-        BracketPair {
-            start: "|".into(),
-            end: "|".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-        BracketPair {
-            start: "[".into(),
-            end: "]".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-        BracketPair {
-            start: "<".into(),
-            end: ">".into(),
-            close: true,
-            surround: true,
-            newline: false,
-        },
-    ]
+fn literal_surround_pair(ch: char) -> Option<SurroundPair> {
+    SURROUND_PAIRS
+        .iter()
+        .find(|p| p.open == ch || p.close == ch)
+        .copied()
 }
 
-fn pair_to_object(pair: &BracketPair) -> Option<Object> {
-    match pair.start.as_str() {
-        "'" => Some(Object::Quotes),
-        "`" => Some(Object::BackQuotes),
-        "\"" => Some(Object::DoubleQuotes),
-        "|" => Some(Object::VerticalBars),
-        "(" => Some(Object::Parentheses),
-        "[" => Some(Object::SquareBrackets),
-        "{" => Some(Object::CurlyBrackets),
-        "<" => Some(Object::AngleBrackets),
-        _ => None,
+/// Resolve a character (including Vim aliases) to its surround pair.
+/// Returns None for 'm' (match nearest) or unknown chars.
+pub fn surround_pair_for_char_vim(ch: char) -> Option<SurroundPair> {
+    let resolved = match ch {
+        'b' => ')',
+        'B' => '}',
+        'r' => ']',
+        'a' => '>',
+        'm' => return None,
+        _ => ch,
+    };
+    literal_surround_pair(resolved)
+}
+
+/// Get a BracketPair for the given string, with fallback for unknown chars.
+/// For vim surround operations that accept any character as a surround.
+pub fn bracket_pair_for_str_vim(text: &str) -> BracketPair {
+    text.chars()
+        .next()
+        .and_then(surround_pair_for_char_vim)
+        .map(|p| p.to_bracket_pair())
+        .unwrap_or_else(|| BracketPair {
+            start: text.to_string(),
+            end: text.to_string(),
+            close: true,
+            surround: true,
+            newline: false,
+        })
+}
+
+/// Resolve a character to its surround pair using Helix semantics (no Vim aliases).
+/// Returns None only for 'm' (match nearest). Unknown chars map to symmetric pairs.
+pub fn surround_pair_for_char_helix(ch: char) -> Option<SurroundPair> {
+    if ch == 'm' {
+        return None;
     }
+    literal_surround_pair(ch).or_else(|| Some(SurroundPair::new(ch, ch)))
+}
+
+/// Get a BracketPair for the given string in Helix mode (literal, symmetric fallback).
+pub fn bracket_pair_for_str_helix(text: &str) -> BracketPair {
+    text.chars()
+        .next()
+        .and_then(surround_pair_for_char_helix)
+        .map(|p| p.to_bracket_pair())
+        .unwrap_or_else(|| BracketPair {
+            start: text.to_string(),
+            end: text.to_string(),
+            close: true,
+            surround: true,
+            newline: false,
+        })
 }
 
 #[cfg(test)]
@@ -1702,4 +1677,41 @@ mod test {
             Mode::Normal,
         );
     }
+
+    #[test]
+    fn test_surround_pair_for_char() {
+        use super::{SURROUND_PAIRS, surround_pair_for_char_helix, surround_pair_for_char_vim};
+
+        fn as_tuple(pair: Option<super::SurroundPair>) -> Option<(char, char)> {
+            pair.map(|p| (p.open, p.close))
+        }
+
+        assert_eq!(as_tuple(surround_pair_for_char_vim('b')), Some(('(', ')')));
+        assert_eq!(as_tuple(surround_pair_for_char_vim('B')), Some(('{', '}')));
+        assert_eq!(as_tuple(surround_pair_for_char_vim('r')), Some(('[', ']')));
+        assert_eq!(as_tuple(surround_pair_for_char_vim('a')), Some(('<', '>')));
+
+        assert_eq!(surround_pair_for_char_vim('m'), None);
+
+        for pair in SURROUND_PAIRS {
+            assert_eq!(
+                as_tuple(surround_pair_for_char_vim(pair.open)),
+                Some((pair.open, pair.close))
+            );
+            assert_eq!(
+                as_tuple(surround_pair_for_char_vim(pair.close)),
+                Some((pair.open, pair.close))
+            );
+        }
+
+        // Test unknown char returns None
+        assert_eq!(surround_pair_for_char_vim('x'), None);
+
+        // Helix resolves literal chars and falls back to symmetric pairs.
+        assert_eq!(
+            as_tuple(surround_pair_for_char_helix('*')),
+            Some(('*', '*'))
+        );
+        assert_eq!(surround_pair_for_char_helix('m'), None);
+    }
 }
