@@ -6,6 +6,7 @@ use gpui::{
 use ui::prelude::*;
 use ui::{h_flex, v_flex};
 
+#[derive(Copy, Clone, Debug, PartialEq)]
 pub enum ToolbarItemEvent {
     ChangeLocation(ToolbarItemLocation),
 }
@@ -109,9 +110,10 @@ impl Render for Toolbar {
         v_flex()
             .group("toolbar")
             .relative()
-            .p(DynamicSpacing::Base08.rems(cx))
+            .py(DynamicSpacing::Base06.rems(cx))
+            .px(DynamicSpacing::Base08.rems(cx))
             .when(has_left_items || has_right_items, |this| {
-                this.gap(DynamicSpacing::Base08.rems(cx))
+                this.gap(DynamicSpacing::Base06.rems(cx))
             })
             .border_b_1()
             .border_color(cx.theme().colors().border_variant)
@@ -119,12 +121,13 @@ impl Render for Toolbar {
             .when(has_left_items || has_right_items, |this| {
                 this.child(
                     h_flex()
-                        .min_h_6()
+                        .items_start()
                         .justify_between()
                         .gap(DynamicSpacing::Base08.rems(cx))
                         .when(has_left_items, |this| {
                             this.child(
                                 h_flex()
+                                    .min_h_8()
                                     .flex_auto()
                                     .justify_start()
                                     .overflow_x_hidden()
@@ -134,17 +137,9 @@ impl Render for Toolbar {
                         .when(has_right_items, |this| {
                             this.child(
                                 h_flex()
-                                    .h_full()
+                                    .h_8()
                                     .flex_row_reverse()
-                                    .map(|el| {
-                                        if has_left_items {
-                                            // We're using `flex_none` here to prevent some flickering that can occur when the
-                                            // size of the left items container changes.
-                                            el.flex_none()
-                                        } else {
-                                            el.flex_auto()
-                                        }
-                                    })
+                                    .when(has_left_items, |this| this.flex_none())
                                     .justify_end()
                                     .children(self.right_items().map(|item| item.to_any())),
                             )
