@@ -2843,3 +2843,84 @@ async fn test_write_file_encoding(cx: &mut gpui::TestAppContext) {
         );
     }
 }
+
+#[gpui::test]
+async fn test_refresh_entries_for_paths_creates_ancestors(cx: &mut TestAppContext) {
+    init_test(cx);
+    let fs = FakeFs::new(cx.background_executor.clone());
+    fs.insert_tree(
+        "/root",
+        json!({
+            "a": {
+                "b": {
+                    "c": {
+                        "deep_file.txt": "content",
+                        "sibling.txt": "content"
+                    },
+                    "d": {
+                        "under_sibling_dir.txt": "content"
+                    }
+                }
+            }
+        }),
+    )
+    .await;
+
+    let tree = Worktree::local(
+        Path::new("/root"),
+        true,
+        fs.clone(),
+        Default::default(),
+        false, // Disable scanning so the initial scan doesn't discover any entries
+        &mut cx.to_async(),
+    )
+    .await
+    .unwrap();
+
+    cx.read(|cx| tree.read(cx).as_local().unwrap().scan_complete())
+        .await;
+
+    tree.read_with(cx, |tree, _| {
+        assert_eq!(
+            tree.entries(true, 0)
+                .map(|e| e.path.as_ref())
+                .collect::<Vec<_>>(),
+            &[rel_path("")],
+            "Only root entry should exist when scanning is disabled"
+        );
+
+        assert!(tree.entry_for_path(rel_path("a")).is_none());
+        assert!(tree.entry_for_path(rel_path("a/b")).is_none());
+        assert!(tree.entry_for_path(rel_path("a/b/c")).is_none());
+        assert!(
+            tree.entry_for_path(rel_path("a/b/c/deep_file.txt"))
+                .is_none()
+        );
+    });
+
+    tree.read_with(cx, |tree, _| {
+        tree.as_local()
+            .unwrap()
+            .refresh_entries_for_paths(vec![rel_path("a/b/c/deep_file.txt").into()])
+    })
+    .recv()
+    .await;
+
+    tree.read_with(cx, |tree, _| {
+        assert_eq!(
+            tree.entries(true, 0)
+                .map(|e| e.path.as_ref())
+                .collect::<Vec<_>>(),
+            &[
+                rel_path(""),
+                rel_path("a"),
+                rel_path("a/b"),
+                rel_path("a/b/c"),
+                rel_path("a/b/c/deep_file.txt"),
+                rel_path("a/b/c/sibling.txt"),
+                rel_path("a/b/d"),
+            ],
+            "All ancestors should be created when refreshing a deeply nested path"
+        );
+    });
+}
