@@ -1,8 +1,10 @@
 use client::{Client, UserStore};
 use codestral::CodestralEditPredictionDelegate;
 use collections::HashMap;
-use copilot::{Copilot, CopilotEditPredictionDelegate};
-use edit_prediction::{SweepFeatureFlag, ZedEditPredictionDelegate, Zeta2FeatureFlag};
+use copilot::CopilotEditPredictionDelegate;
+use edit_prediction::{
+    MercuryFeatureFlag, SweepFeatureFlag, ZedEditPredictionDelegate, Zeta2FeatureFlag,
+};
 use editor::Editor;
 use feature_flags::FeatureFlagAppExt;
 use gpui::{AnyWindowHandle, App, AppContext as _, Context, Entity, WeakEntity};
@@ -163,7 +165,14 @@ fn assign_edit_prediction_provider(
             editor.set_edit_prediction_provider::<ZedEditPredictionDelegate>(None, window, cx);
         }
         EditPredictionProvider::Copilot => {
-            if let Some(copilot) = Copilot::global(cx) {
+            let ep_store = edit_prediction::EditPredictionStore::global(client, &user_store, cx);
+            let Some(project) = editor.project().cloned() else {
+                return;
+            };
+            let copilot =
+                ep_store.update(cx, |this, cx| this.start_copilot_for_project(&project, cx));
+
+            if let Some(copilot) = copilot {
                 if let Some(buffer) = singleton_buffer
                     && buffer.read(cx).file().is_some()
                 {
@@ -202,9 +211,11 @@ fn assign_edit_prediction_provider(
                         } else if name == EXPERIMENTAL_ZETA2_EDIT_PREDICTION_PROVIDER_NAME
                             && cx.has_flag::<Zeta2FeatureFlag>()
                         {
-                            edit_prediction::EditPredictionModel::Zeta2
+                            edit_prediction::EditPredictionModel::Zeta2 {
+                                version: Default::default(),
+                            }
                         } else if name == EXPERIMENTAL_MERCURY_EDIT_PREDICTION_PROVIDER_NAME
-                            && cx.has_flag::<Zeta2FeatureFlag>()
+                            && cx.has_flag::<MercuryFeatureFlag>()
                         {
                             edit_prediction::EditPredictionModel::Mercury
                         } else {
